#include "u_plot.h"
#include "u_pl_x11.h"
#include "u_pl_loc.h"
#include "v_inp.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include <X11/Xlib.h>
#include <X11/cursorfont.h>
#include <X11/Xutil.h>


#ifdef MOTIF
#include "x_inplib.h"
#else
#endif
#define FontName_6 "-schumacher-clean-medium-r-normal--6-60-75-75-c-40-iso646.1991-irv"
#define FontName_7 "-misc-fixed-medium-r-normal--7-70-75-75-c-50-iso8859-1"
#define FontName_10 "-misc-fixed-medium-r-normal--10-100-75-75-c-60-iso8859-1"
#define FontName_16 "-sony-fixed-medium-r-normal--16-120-100-100-c-80-iso8859-1"
#define FontName_20 "-misc-fixed-medium-r-normal--20-200-75-75-c-100-iso8859-1"


#define WindowName "Graphic Display"

#define COLENV "PLOTX11COLORS"
#define COLFILE "plotx11colors.txt"

#define DEBUG FALSE

/* Default colors, used if color file not found */
#define MAXCOLOR  9
   static char colors[MAXCOLOR][30] = {
           "white",
           "black",
           "blue",
           "red",
           "green",
           "orange",
           "cyan",
           "magenta",
           "yellow"};

/* We use arrays instead of dynamic allocation; on a XWindow-System */
/* some KB do not matter                                            */

static Display *display = 0;
static input_active[MAXHANDLES];
static Window windows[MAXHANDLES];                        /* Windows         */ 
static GC gc[MAXHANDLES];                                 /* Graphic Context */
static Screen *screen = NULL;                                    /* Screen          */
static unsigned int dispsizex, dispsizey;
                 /* Size of Screen  */
static int devsizey[MAXHANDLES];    /* Size of Window  */
static int foreground[MAXHANDLES], background[MAXHANDLES];/* Colours         */ 
static int text_xalign[MAXHANDLES],text_yalign[MAXHANDLES];/*Text alignment  */

/* variables for save of input focus */
static Window save_focus[MAXHANDLES];                      
static int save_revert[MAXHANDLES], is_graphmode[MAXHANDLES];

/* Default properties of windows */
#define DEFWIN_W    400
#define DEFWIN_H    300
#define DEFWIN_X     10
#define DEFWIN_Y     10
#define DEFBORDER     5
#define DEFFOREGR     1
#define DEFBACKGR     0

#ifdef VMS
   void lib$wait(float *t);
#endif    

int plotd_sel_font(int nh,int sizey){
   static Font font;
   XFontStruct *xf;
   int i,j,iz;
   j=1;
   iz=sizey;
   nofont:
   if(iz>0){
   i=sizey/100;
   if(i<2) i=2;if(i>6) i=6; iz=0;
   }else{i=j;}
 
   switch(i){
     case 2: xf=XLoadQueryFont(display, FontName_6);
             if(xf>0) font = XLoadFont(display, FontName_6);break;
     case 3: xf=XLoadQueryFont(display, FontName_7);
             if(xf>0) font = XLoadFont(display, FontName_7);break;
     case 4: xf=XLoadQueryFont(display, FontName_10);
             if(xf>0) font = XLoadFont(display, FontName_10);break;
     case 5: xf=XLoadQueryFont(display, FontName_16);
             if(xf>0) font = XLoadFont(display, FontName_16);break;
     case 6: xf=XLoadQueryFont(display, FontName_20);
             if(xf>0) font = XLoadFont(display, FontName_20);break;
   };
   j=j+1;
   if(xf <= 0 && j<7) goto nofont;
   if(xf <= 0 && j==7) printf(" the following fonts are defined in u_pl_x11.c 6_=%s \n  7_=%s \n 10_=%s \n 16_=%s \n 20_=%s \n",
                        FontName_6,FontName_7,FontName_10,FontName_16,FontName_20);
   if(j==2) {j=PLOT_STAT_OK;
   XSetFont(display, gc[nh], font);}else{j=10;}
   return(j);
}

/*************************** Local routines ***************************/
/* Remove all events pending for this window, we don't want to redraw */
/* more than once                                                     */
static void RemovePendingEvents(int handle){
    XEvent event;
#ifdef VMS
    /* Wait a little bit, so we can catch e.g. all expose-events */
    /* this may save a complete redraw...                        */
    static float t = 0.5;
    lib$wait(&t);
#endif    
    do{} while(XCheckWindowEvent(display, windows[handle],
			      ButtonPressMask | KeyPressMask | ExposureMask,
			      &event));
}

static int doKeyPress(XKeyEvent ep, int *x, int *y)
/* Handle Keyboard Events */
{   char buffer;
    int nchars;
    int x_root,y_root;
    /* Get character code from event */
    nchars = XLookupString(&ep,&buffer,1,NULL,NULL);
/*    nchars = XLookupString((XKeyEvent*)&ep,&buffer,20,NULL,NULL);*/
    *x = ep.x; *y = ep.y; x_root = ep.x_root; y_root = ep.y_root;
    /*printf("doKeyPress xroot=%d yroot=%d \n",x_root,y_root);*/
       if (nchars==1) return buffer; /* normal key --> char */
                 return 0;                     /* other key  -->    0 */
    
}
static int doButtonPress(XButtonEvent ep, int *x, int *y)
/* Handle Mouse Key and   Keyboard Events */
{   char buffer;
    int nchars;
    int x_root,y_root;
    /* Get character code from event */
 nchars = XLookupString((XKeyEvent*)&ep,&buffer,1,NULL,NULL);
    *x = ep.x; *y = ep.y; x_root = ep.x_root; y_root = ep.y_root;
    /*printf("doButtonPress xroot=%d yroot=%d \n",x_root,y_root);*/
    switch (ep.button){ /* Mouse buttons 1,2,3 */
       case 1 : return(PLOT_TRM_MB_LEFT); break;
       case 2 : return(PLOT_TRM_MB_CENTER); break;
       case 3 : return(PLOT_TRM_MB_RIGHT); break;
       /* No mouse button */
       default : if (nchars==1) return buffer; /* normal key --> char */
                 return 0;                     /* other key  -->    0 */
    }
}

int doDefineColor(int n){
/* Allocate a color by name                       */
/* For color names, see e.g. /usr/lib/X11/rgb.txt */
   int pixel;
   XColor exact_color,screen_color;
   if ((n < 0) || (n >= MAXCOLOR)){ 
      if (DEBUG) printf(" Colour out of range: %d\n",n);
      return XBlackPixelOfScreen(screen); }
   if ((XDefaultVisualOfScreen(screen))->class == TrueColor
       || (XDefaultVisualOfScreen(screen))->class == PseudoColor
       || (XDefaultVisualOfScreen(screen))->class == DirectColor
       || (XDefaultVisualOfScreen(screen))->class == StaticColor){
       if (XAllocNamedColor(display, XDefaultColormapOfScreen(screen),
	   colors[n], &screen_color, &exact_color)){
           if (DEBUG) printf("Named color allocated: %s\n",colors[n]);
           return screen_color.pixel;}
       else printf("Named color failed: <<%s>>\n",colors[n]);
   }
   else  if (DEBUG) printf("No colour system\n");
   if (n == 0){
      if (DEBUG) printf(" Returning White Pixel\n");
      return XWhitePixelOfScreen(screen);}
   if (DEBUG) printf(" Returning Black Pixel\n");
   return XBlackPixelOfScreen(screen);
}


/**************** End of Local routines **************/

int plotd_modify_X11(int nh,/* handletyp *h,modif. GA:31.05.1995*/ int sizex, int sizey,
  		     int posx, int posy, char name[]){
/* Change window size, position or title */ 
    XSizeHints xsh;

    if (name) XStoreName(display, windows[nh], name);
    xsh.flags = 0;
    if ((sizex > 0) && (sizey > 0)){
         xsh.width = sizex;   xsh.height = sizey;
         xsh.flags = xsh.flags | PSize; }
    if ((posx > 0) && (posy > 0)){
	 xsh.x = posx;    xsh.y = posy;
         xsh.flags = xsh.flags | PPosition; }
    if (xsh.flags) XSetNormalHints(display, windows[nh], &xsh);
    return 1;
}

int plotd_getdevice_X11(int handle,int *posx,int *posy,int *sizex,int *sizey){
/* get actual size of window */
   XWindowAttributes xx;
   unsigned int borderw,depth;
   Window rootwin;
   int px,py;
 XGetWindowAttributes(display,windows[handle],&xx);
 (void) XTranslateCoordinates (display, windows[handle], 
                                xx.root, 
				-xx.border_width,
				-xx.border_width,
				posx,posy, &rootwin);
/*  printf(
"2plotd_getdevice_X11  xo=%d yo=%d  width=%d height=%d \n",
*posx,*posy,xx.width,xx.height);*/

 /* XGetGeometry(display,windows[handle],&rootwin,
                &px,&py,
        	(unsigned*)sizex,(unsigned*)sizey,&borderw,&depth);*/
 devsizey[handle]=xx.height;
 *sizex=xx.width;
 *sizey=xx.height;
   return(PLOT_DEV_X11);
}


void plotd_line_X11(int handle, int X0, int Y0, int X1, int Y1){
/* Draw a line */
   XDrawLine(display,windows[handle],gc[handle],X0,devsizey[handle]-Y0,
	                                    X1,devsizey[handle]-Y1);
}

void plotd_point_X11(int handle, int X, int Y){
/* Plot a point, i.e. a line of length 0 */ 
   plotd_line_X11(handle,X,Y,X,Y);
}

void plotd_polyline_X11(int handle, int pts, int X[], int Y[]){
/* Draw a polyline */
     short int i;
     XPoint *itmp;
     itmp = malloc(2*pts*sizeof(short int));
     for (i=0; i<pts;i++){
	 itmp[i].x=X[i]; itmp[i].y=devsizey[handle]-Y[i]; }
     XDrawLines(display,windows[handle],gc[handle],itmp,pts,CoordModeOrigin);
     free(itmp);
}

int plotd_setfg_X11(int handle, int fg){
/* set foreground color */
    XGCValues xgcv;
    if (fg==PLOT_USEDEFAULT) fg = DEFFOREGR;
    fg = fg % MAXCOLOR;
    if (fg==background[handle]) fg = DEFFOREGR;
    foreground[handle] = fg;
    XSetForeground(display,gc[handle],doDefineColor(fg));
    return(foreground[handle]);
}

int plotd_getfg_X11(int handle){
/* get foreground color */
   return(foreground[handle]);
}

int plotd_setbg_X11(int handle, int bg){
/* set background color */
    XGCValues xgcv;
    if (bg==PLOT_USEDEFAULT) bg = DEFBACKGR;
    bg = bg % MAXCOLOR;
    xgcv.background = doDefineColor(bg);
    background[handle] = bg;
    XChangeGC(display, gc[handle], GCBackground, &xgcv);
    return(background[handle]);
}

int plotd_getbg_X11(int handle){
/* get background color */
    return(background[handle]);
}

/* On colour X-window systems, only REPLACE (= GXcopy) can be used, */
/* since colours are indices in the colour table                    */
int plotd_setmode_X11(int handle, int newmode){
   return PLOT_MODE_REPLACE;
}    

int plotd_getmode_X11(int handle){
   return PLOT_MODE_REPLACE;
}

static void TextExtent(int handle, char txt[], int *sizex, 
                       int *ascent,int *descent){
    
/* Get text extent from server: length, ascent, descent */ 
    int dirret, asc, desc;
    XCharStruct overall;
    XGCValues gcval;
    unsigned int mask;

    mask=GCFont;
    XGetGCValues(display,gc[handle],mask,&gcval);
    XQueryTextExtents(display,gcval.font,txt,strlen(txt),
                      &dirret,&asc,&desc,&overall);
    *sizex = overall.width;
    *ascent = overall.ascent;
    *descent = overall.descent;
}

void plotd_writetext_X11(int handle, int x, int y, char txt[]){
/* Text output at (x,y) using actual alignment */
    int sizex, sizey, desc, asc;
    TextExtent(handle,txt,&sizex,&asc,&desc);
    sizey = asc + desc;
    if (text_xalign[handle]==PLOT_RIGHTTEXT)  x = x - sizex;
    if (text_xalign[handle]==PLOT_CENTERTEXT) x = x - sizex/2;
    if (text_yalign[handle]==PLOT_CENTERTEXT) y = y + desc - sizey/2;
    if (text_yalign[handle]==PLOT_TOPTEXT)    y = y - asc;
    if (text_yalign[handle]==PLOT_BOTTOMTEXT) y = y + desc;
    XDrawString(display, windows[handle], gc[handle],
		x, devsizey[handle]-y, txt, strlen(txt));
}

void plotd_erasetext_X11(int handle, int x, int y, char txt[]){
/* erase text at (x,y) using actual alignment. This can only be done */
/* via XClearArea.                                                   */
    int sizex, sizey, desc, asc;
    TextExtent(handle,txt,&sizex,&asc,&desc);
    sizey = asc + desc;
    if (text_xalign[handle]==PLOT_RIGHTTEXT)  x = x - sizex;
    if (text_xalign[handle]==PLOT_CENTERTEXT) x = x - sizex/2;
    if (text_yalign[handle]==PLOT_CENTERTEXT) y = y + desc - sizey/2;
    if (text_yalign[handle]==PLOT_TOPTEXT)    y = y - asc;
    if (text_yalign[handle]==PLOT_BOTTOMTEXT) y = y + desc;
    XClearArea(display, windows[handle], x, devsizey[handle]-y-asc,
               sizex, sizey, FALSE);
}

void plotd_focus_X11(int handle){
/* Switch to graphic mode: bring graphic window to top, save input focus, */
/* give input focus to graphic window, and remove (expose-) events; we    */
/* know that we want to redraw.                                           */
    XRaiseWindow(display,windows[handle]);
    XGetInputFocus(display,&save_focus[handle],&save_revert[handle]);
    XSetInputFocus(display,windows[handle],save_revert[handle],CurrentTime);
  /*XSetInputFocus(display,windows[handle],RevertToPointerRoot,CurrentTime);*/
    RemovePendingEvents(handle);
}

void plotd_graphmode_X11(int handle){
/* Switch to graphic mode: bring graphic window to top, save input focus, */
/* give input focus to graphic window, and remove (expose-) events; we    */
/* know that we want to redraw.                                           */
    if (is_graphmode[handle]) return;
    XRaiseWindow(display,windows[handle]);
    XGetInputFocus(display,&save_focus[handle],&save_revert[handle]);
    /*XSetInputFocus(display,windows[handle], RevertToParent,CurrentTime);*/
    RemovePendingEvents(handle);
    text_xalign[handle] = PLOT_LEFTTEXT;
    text_yalign[handle] = PLOT_BOTTOMTEXT;
    is_graphmode[handle] = TRUE;
}

void plotd_clear_X11(int handle){
/* Clear graphic window */
    XClearWindow(display, windows[handle]);
}

void plotd_textmode_X11(int handle){
/* Switch to textmode: Lower graphic window, return input focus to terminal */
    if (! is_graphmode[handle]) return;
    /*    XLowerWindow(display,windows[handle]);*/
    XSetInputFocus(display,save_focus[handle],save_revert[handle],CurrentTime);
    is_graphmode[handle] = FALSE;
}

void plotd_close_X11(int handle){
/* Destroy a graphic window */
    XFreeGC(display,gc[handle]);
    XUnmapWindow(display,windows[handle]);
    XDestroyWindow(display,windows[handle]);
}

int plotd_getpointer_X11(int *handle,int mhandle, int *x, int *y){
/* Wait for event, return pointer position and terminator */
/* Terminator is: - key for keypress event                */
/*                - Mouse button for button event         */
/*                - PLOT_TRM_REPLOT for expose event      */
/* position is only valid for keypress and button events  */ 

   XEvent event;
   int trm,i;
   /*if (input_active[handle]) return PLOT_TRM_ALREADY_ACTIVE;*/
   do { 
      XNextEvent(display, &event);
      for(i=0;i<mhandle;i++){
       if (event.xany.window == windows[i]){
       *handle=i;input_active[i] = TRUE;
       /*	 continue;*/
       }}
      switch (event.type) {
        case Expose: RemovePendingEvents(i);
                      input_active[i] = FALSE;
		      /* printf("plotd_getpointer_X11 expose handle %d \n",i);*/
	              return(PLOT_TRM_REPLOT);
	 case KeyPress   : trm = doKeyPress(event.xkey,x,y); break; 
         case ButtonPress: trm = doButtonPress(event.xbutton,x,y); break;
  	 default : ;
   }} while(trm == 0);
   *y = devsizey[*handle] - *y;
   input_active[*handle] = FALSE;
   return(trm);
}

int plotd_checkevent_X11(int handle, int *x, int *y){
/* Check for event, return pointer position and terminator */
/* Terminator is: - key for keypress event                 */
/*                - Mouse button for button event          */
/*                - PLOT_TRM_REPLOT for expose event       */
/*                - PLOT_TRM_ABORT if no event present     */
/* position is only valid for keypress and button events   */ 
   XEvent event;
   int trm;
   trm = 0;
   if (XCheckWindowEvent(display, windows[handle],
		     ButtonPressMask | KeyPressMask | ExposureMask,
		     &event)){
      switch (event.type) {
	 case Expose: RemovePendingEvents(handle);
	              return(PLOT_TRM_REPLOT);
	 case KeyPress   : trm = doKeyPress(event.xkey,x,y); break; 
         case ButtonPress: trm = doButtonPress(event.xbutton,x,y); break;
  	 default : ;
      }
      /*   printf("plotd_getpointer_X11 x=%d y=%d\n",*x,*y);*/
       
      *y = devsizey[handle] - *y;
      return(trm);}
   return PLOT_TRM_ABORT;
}

static void plotd_settextalign_X11(int handle, int xalign, int yalign){
/* set text alignment */
   text_xalign[handle] = xalign;
   text_yalign[handle] = yalign;
}

static void ReadColorNames(){
/* Read color names from file */
   int i;
   char *p;
   FILE *cf;
   char fname[80], *tmp;
   if ((tmp=getenv(COLENV))!=NULL) strcpy(fname,tmp);
   else strcpy(fname,COLFILE);
   if (!(cf=fopen(fname,"r"))) return;
   for (i=0; i<MAXCOLOR; i++){
      fgets(colors[i],30,cf);
      p = strchr(colors[i],10);
      if (p != 0) *p = 0;}
   fclose(cf);
}
      
void plotd_polysymbol_X11(int handle, int pts, symboltyp sym,int x[], int y[]){
   int i,k, nseg;
   XSegment *seg;
   nseg = pts * sym.vec;			     
   seg = malloc(nseg*sizeof(XSegment));			     
   for (i=0; i<pts; i++)
     for(k=0; k<sym.vec; k++){				  
       seg[sym.vec*i+k].x1 = x[i]+sym.x1[k];			     
       seg[sym.vec*i+k].x2 = x[i]+sym.x2[k];			     
       seg[sym.vec*i+k].y1 = devsizey[handle]-(y[i]+sym.y1[k]);	     
       seg[sym.vec*i+k].y2 = devsizey[handle]-(y[i]+sym.y2[k]);
     }
   XDrawSegments(display,windows[handle],gc[handle],seg,nseg);
   free(seg);			     		     
}
			     
int plotd_init_x11(int nh, handletyp *h, int sizex, int sizey,
		     int posx, int posy, char name[]){
/* Create a plotwindow, initialize X-Window graphic if it is the first one */
   XGCValues xgcv;
   XEvent event;
   XSizeHints xsh;
   XSetWindowAttributes xswa;
   static Font font;
   int dummy;
   unsigned udummy;
   int xdummy, ydummy;
   Window dummywin;
   Cursor the_cursor;

   /* If no display defined, create Display */
   if (display==NULL){
#ifdef MOTIF
      if (MOTIF_interface_enabled()) display = MOTIF_get_main_display();
      else
#endif
      display = XOpenDisplay(0);
   }
   if (display==NULL) return(PLOT_STAT_NODISPLAY);
   if (screen == NULL){    			
      screen = XDefaultScreenOfDisplay(display);
      XGetGeometry(display,XRootWindowOfScreen(screen),&dummywin,&dummy,&dummy,
        	       &dispsizex,&dispsizey,&udummy,&udummy);
      XSynchronize(display, 1);
      ReadColorNames();
   }
   /* Save input focus, so we can give it back to terminal */
   XGetInputFocus(display,&save_focus[nh],&save_revert[nh]);
   xswa.event_mask = ExposureMask | ButtonPressMask | KeyPressMask;
   xswa.background_pixel = doDefineColor(0);
   xswa.backing_store    = WhenMapped;

   /*printf("plotd_init_x11 px=%d py=%d devx=%d devy=%d\n",
     posx,posy,dispsizex,dispsizey);*/

   windows[nh] = XCreateWindow(display, XRootWindowOfScreen(screen),
		 posx, posy,sizex,sizey, DEFBORDER,
		 XDefaultDepthOfScreen(screen), InputOutput,
		 XDefaultVisualOfScreen(screen),
		 CWEventMask | CWBackPixel | CWBackingStore, &xswa);
   the_cursor = XCreateFontCursor(display,XC_crosshair);
   XDefineCursor(display,windows[nh],the_cursor);
   /*  XResizeWindow(display,windows[nh], sizex, sizey);*/

/*   xgcv.foreground = doDefineColor(DEFFOREGR);
   xgcv.background = doDefineColor(DEFBACKGR);*/
   xgcv.foreground = BlackPixel(display,DefaultScreen(display));
   xgcv.background = WhitePixel(display,DefaultScreen(display));
   xgcv.function = GXcopy;      
   xgcv.plane_mask = AllPlanes; 
   gc[nh] = XCreateGC(display, windows[nh],
		  GCFunction | GCForeground | GCBackground | GCPlaneMask, 
		  &xgcv);
   dummy=plotd_sel_font(nh,sizey);
   xsh.x = posx;
   xsh.y = posy;
   xsh.width  = sizex;
   xsh.height = sizey;
   xsh.flags = PPosition | PSize;
   XSetNormalHints(display, windows[nh], &xsh);
   XStoreName(display, windows[nh], name);
   XMapWindow(display, windows[nh]);
   XWindowEvent(display,windows[nh], ExposureMask, &event);
   RemovePendingEvents(nh);
   plotd_getdevice_X11(nh,&posx,&posy,&xdummy,&ydummy);

   h->ploth_modify     =   plotd_modify_X11;
   h->ploth_getdevice  =   plotd_getdevice_X11;
   h->ploth_line       =   plotd_line_X11;
   h->ploth_point      =   plotd_point_X11;
   h->ploth_polyline   =   plotd_polyline_X11;
   h->ploth_setfg      =   plotd_setfg_X11;
   h->ploth_getfg      =   plotd_getfg_X11;
   h->ploth_setbg      =   plotd_setbg_X11;
   h->ploth_getbg      =   plotd_getbg_X11;
   h->ploth_writetext  =   plotd_writetext_X11;
   h->ploth_erasetext  =   plotd_erasetext_X11;
   h->ploth_graphmode  =   plotd_graphmode_X11;
   h->ploth_clear      =   plotd_clear_X11;
   h->ploth_textmode   =   plotd_textmode_X11;
   h->ploth_close      =   plotd_close_X11;
   h->ploth_setmode    =   plotd_setmode_X11;
   h->ploth_getmode    =   plotd_getmode_X11;
   /* h->ploth_getpointer =   plotd_getpointer_X11;*/
   h->ploth_checkevent =   plotd_checkevent_X11;
   h->ploth_settextalign = plotd_settextalign_X11;
   h->ploth_polysymbol   = plotd_polysymbol_X11;
   h->capabilities = PLOT_CAPABILITY_POINTER + PLOT_CAPABILITY_EVENT 
		   + PLOT_CAPABILITY_WINDOWS + PLOT_CAPABILITY_TEXTEXT;
   text_xalign[nh] = PLOT_LEFTTEXT;
   text_yalign[nh] = PLOT_BOTTOMTEXT;
   input_active[nh] = FALSE;		
   is_graphmode[nh] = TRUE;
   plotd_textmode_X11(nh);
   if(dummy>0) {return(dummy);}else{return(PLOT_STAT_OK);}
}

