#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>

#include "v_inp.h"
#include "u_plot.h"
#include "v_plotstr.h"
#include "xmosplo.h"
#include "u_pl_loc.h"
#include "u_plot1.h"

#define FRAME 5      /* pixel frame */
#define TICS 11
#define TICLEN 3
#define TEXTHEIGHT 15       /* pixel */
#define TOP_FRAME 0.94
#define BOTTOM_FRAME 0.15
#define LEFT_FRAME 0.18
#define RIGHT_FRAME 0.95
#define INFO_POSX ((RIGHT_FRAME+LEFT_FRAME)/2)
#define TITLE_POSY (BOTTOM_FRAME*0.1)
#define XLABEL_POSY (BOTTOM_FRAME*0.2)
#define YLABEL_POSY (TOP_FRAME*0.9)
#define Y2LABEL_POSY (TOP_FRAME*0.85)

#define VHIGH 1.0e38
#define DEFAULTFORMATX "%6.2f"
#define DEFAULTFORMATY "%10.1f"

static int status;
static int symbols_loaded = 0;

char plotmessages[12][40] = {"ok",
                            "No more handles",
                            "Out of memory",
                            "No such handle",
                            "No more handles of this type",
                            "Can't open Display",
                            "No such device",
                            "Error opening outputfile",
                            "Symbolfile not found",
                            "Graphic Driver not found",
                            "none of these fonts available"};

/*=========== Init/Exit plot package ===============*/

#define SYMFILE "MODEL.SYM"
#define SYMENV  "EFFIUTPATH"
#define MAXSYMBOLS 34

char wdescription[80]="EFFI plotwindow";

static struct plotstruct plc[MAXHANDLES];
static int is_window_device;

static handleptr handles[MAXHANDLES];
static double size_factor; 

plotstruct *get_plotconfig(){
   return(&plc[MAXHANDLES]);
}

double sqrt(double x);


static int  frame, ticlen, sizex, sizey;
static int a_h = -1;
static int mhandle =0;
static int do_auto_update = FALSE;

static int wksp[MAXHANDLES+1];
static int n_sub, n_pts, n_data, n_theo;

static int first_channel, last_channel;
static int have_subspectra, plot_subspectra, pl_subsp[MAXSUB],have_theo, 
           plot_data, plot_theo, plot_auto,
           winsizex, winsizey,winposx,winposy;


/*=====================================================================*/
/* Universal plot routines for various devices                         */
/* V2.0                    by A.Vef                           08.12.93 */
/*=====================================================================*/
/* How it works:                                                       */
/*  - this module provides device-independent routines. A call to      */
/*    plot_open() connects them to the device-specific implementation  */
/*  - Method: pointers to the functions                                */
/* How to add a new driver:                                            */
/*  - define an appropriate constant PLOT_DEV_xxx in U_PL_DEF.H        */
/*  - write the driver according to the examples.                      */
/*     = all routines should be named plotd_<name>_<devtype>(),        */
/*       e.g. plotd_clear_tek()                                        */
/*     = you have to provide ALL routines, even if they are meaning-   */
/*       less for this device (e.g. input on plotter), except the      */
/*       following:                                                    */
/*     = Optional Routines:                                            */
/*         plotd_getpointer_...()                                      */
/*         plotd_checkevent_...()                                      */
/*       their presence is checked at runtime, if they do not exist,   */
/*       the calls are ignored.                                        */
/*         plotd_polysymbol_...()                                      */
/*       checked at runtime, if not present, symbols are plotted using */
/*       plotd_line_...().                                             */
/*  - add the call to plotd_init_<devtype>() to plot_open()            */
/*    if the device is limited to a specific environment, use "#ifdef" */
/*=====================================================================*/


struct symboltyp symbols[MAXSYMBOLS];

void load_sym(){
   int i,j;
   char *tmp,fname[80],instr[80];
   double out[16];
   FILE *symfile;

   if (symbols_loaded) return;
   if ((tmp=getenv(SYMENV))!=NULL){ strcpy(fname,tmp);strcat(fname,"/");
   strcat(fname,SYMFILE);}
   else strcpy(fname,SYMFILE);
   /*   printf("load_sym3 %s\n",fname);*/
   if ((symfile=fopen(fname,"r"))==NULL) return;
   for (i=1;i<MAXSYMBOLS;i++){
      fgets(instr,70,symfile);
      trans(instr,out); symbols[i].vec=out[0];
      symbols[i].x1 = malloc(symbols[i].vec*sizeof(symbolcoordtyp));
      symbols[i].x2 = malloc(symbols[i].vec*sizeof(symbolcoordtyp));
      symbols[i].y1 = malloc(symbols[i].vec*sizeof(symbolcoordtyp));
      symbols[i].y2 = malloc(symbols[i].vec*sizeof(symbolcoordtyp));
      for (j=0; j<symbols[i].vec; j++)
	{ fgets(instr,70,symfile);
          trans(instr,out);
	  symbols[i].x1[j] = out[0];
	  symbols[i].y1[j] = out[1];
	  symbols[i].x2[j] = out[2];
	  symbols[i].y2[j] = out[3];
	}
   }
   fclose(symfile);
   symbols_loaded = TRUE;
}


int plot_init(){
   int i;
   for (i=0; i<MAXHANDLES; i++) handles[i] = NULL;
   load_sym();
   if (symbols_loaded) status = PLOT_STAT_OK;
                 else  status = PLOT_STAT_NOSYMBOLS;
   return(symbols_loaded);
}

void plot_exita(){
   int i;
   for (i=0; i<MAXHANDLES; i++) if (handles[i]){
       plot_close(i); handles[i] = 0;
       wptr.intptr[i]=0;}
}

void plot_exit(int i){
   if (handles[i]){
        plot_close(i); handles[i] = 0;
        wptr.intptr[i]=0;}
}


void calculate_transformation(int handle){
   handleptr h;
   h = handles[handle];
#define scale_one(x1, x2, i1, i2, f, o){\
   int i; double d;\
   i = i2 - i1;     d = x2 - x1;\
   f = d/i;         o = x1 - f * i1;}
   scale_one(h->xwmin,h->xwmax,h->xvmin,h->xvmax,h->xfact,h->xoffs);
   scale_one(h->ywmin,h->ywmax,h->yvmin,h->yvmax,h->yfact,h->yoffs);
   /* printf("h->xfact= %f h->xoffs= %f \n",h->xfact,h->xoffs);*/
#undef scale_one
}


/* convert screen to world coordinates */
void screen_to_world(int handle, int xi, int yi, double *xr, double *yr)
{
   *xr = handles[handle]->xfact * xi + handles[handle]->xoffs;
   *yr = handles[handle]->yfact * yi + handles[handle]->yoffs;
}
/* convert world to screen coordinates */
void world_to_screen(int handle, double xr, double yr, int *xi, int *yi)
{
   *xi = (xr - handles[handle]->xoffs) / handles[handle]->xfact;
   *yi = (yr - handles[handle]->yoffs) / handles[handle]->yfact;
}


/* Open a plot handle, i.e. Window, Plotterfile,... */
/* Returns -1 on error, otherwise handle             */
/* Name is Windowtitle or filename for Plotter      */
int plot_open(int handle,int devicetyp, 
            int sizex, int sizey, int posx, int posy,char name[])
{int i;
#ifdef UNIX
   if (devicetyp==PLOT_DEV_DEFAULT) devicetyp = PLOT_DEV_TEK;
#endif
#ifdef GNUPC
   if (devicetyp==PLOT_DEV_DEFAULT) devicetyp = PLOT_DEV_GNUPC;
#endif
#ifdef BORLANDC
   if (devicetyp==PLOT_DEV_DEFAULT) devicetyp = PLOT_DEV_BGI;
#endif
   if (devicetyp==PLOT_DEV_DEFAULT) devicetyp = PLOT_DEV_NO;
   if ((devicetyp<=PLOT_DEV_NO) || (devicetyp>=PLOT_DEV_LASTDEV)) return(-1);

   for(i=0;i<MAXHANDLES;i++){handles[i] = (handleptr)wptr.intptr[i];}

   /*handle = 0;*/
   while ((handles[handle]) && (handle <MAXHANDLES)) handle++;
   if (handle==MAXHANDLES){ status = PLOT_STAT_NOMOREHANDLES; return(-1);}
   handles[handle] = malloc(sizeof(handletyp));
   if (handles[handle]==NULL){ status = PLOT_STAT_NOMEMORY; return(-1);}
   handles[handle]->devtype = devicetyp;
   /* Initialize optional capabilities to NULL, so we can check */
   /* if they are supported.                                    */
   handles[handle]->ploth_checkevent=NULL;
   handles[handle]->ploth_getpointer=NULL;
   handles[handle]->ploth_polysymbol=NULL;
   handles[handle]->ploth_rectangle=NULL;
   handles[handle]->capabilities=PLOT_CAPABILITY_NONE;		 
   status = plotd_init_x11(handle,handles[handle],
                                      sizex,sizey,posx,posy,name); 
   if (status==PLOT_STAT_OK){
      (*handles[handle]->ploth_getdevice)(handle,
				       &handles[handle]->xposw,
				       &handles[handle]->yposw,
				       &handles[handle]->xvmax,
				       &handles[handle]->yvmax);
      handles[handle]->xwmin=0;      handles[handle]->xwmax=1;
      handles[handle]->ywmin=0;      handles[handle]->ywmax=1;
      handles[handle]->xvmin=0;
      handles[handle]->yvmin=0;
      handles[handle]->akt_x=0;      handles[handle]->akt_y=1;
      calculate_transformation(handle);
      handles[handle]->clip=FALSE;
      handles[handle]->symbol_factor = 1;
      return(handle);
   }else{
      free(handles[handle]); handles[handle] = NULL; return(-1);
   }
}

/* return to textmode:                         */
/*  - stdin/stdout available for text output   */
/*  - graphic output prohibited                */
void plot_textmode(){
   int i;
   for (i=0; i<MAXHANDLES; i++)
     if (handles[i]) (*handles[i]->ploth_textmode)(i);
}

/* switch to graphic mode                                  */
/*  - inquire actual size for each open handle (XWindow !) */
/*  - allow graphic output                                 */
void plot_graphmode(){
   int i;
   for (i=0; i<MAXHANDLES; i++){
      if (handles[i]) (*handles[i]->ploth_graphmode)(i);
   }
}

/* return last error message */
char *plot_message(){
   return plotmessages[status];
}


/*===================== Handle =====================*/
/* Modify existing handle, returns status */
int plot_modify(int handle, int sizex, int sizey, int posx, int posy,
   	        char name[]){
   if (! handles[handle]){status=PLOT_STAT_NOSUCHHANDLE; return(FALSE);}
   status = (*handles[handle]->ploth_modify)(handle,sizex,sizey,posx,posy,name);
   return (status==PLOT_STAT_OK);
}


/* close a handle */
void plot_close(int handle){
   if (! handles[handle]) return;
   (*handles[handle]->ploth_close)(handle);
   free(handles[handle]); handles[handle] = 0;
}

/* Clear a handle: Clear Screen or Window, FormFeed on Plotter */
void plot_clear(int handle){
   if (! handles[handle]) return;
   (*handles[handle]->ploth_clear)(handle);
}


/*===================== Inquire Status ========================*/
/* Status of handle, or general status if handle==0 */
int plot_status(int handle){
   if (handle==0) return(status);
   if (handles[handle]) return(PLOT_STAT_OK);
   return(PLOT_STAT_NOSUCHHANDLE);
}

/* return devicetype and dimensions */
int plot_getdev(int handle,int *posx,int *posy,int *sizex,int *sizey){
   if (! handles[handle]) { status = PLOT_STAT_NOSUCHHANDLE;
      return(PLOT_DEV_NO);   }
   (*handles[handle]->ploth_getdevice)(handle,posx,posy,sizex,sizey);

   return( handles[handle]->devtype);
}

unsigned long plot_capabilities(int handle, unsigned long capability){
   if (capability==PLOT_CAPABILITY_ALL) 
      return handles[handle]->capabilities;
   return ((handles[handle]->capabilities & capability) != 0);
}
   
/*====================== Modify Parameters ====================*/
/* All routines return the value actually used, which may be   */
/* different from the value desired                            */

/* Set foreground colour index */
int plot_setfg(int handle, int foreground){
  return((*handles[handle]->ploth_setfg)(handle,foreground));}

/* Set background colour index */
int plot_setbg(int handle, int background){
  return((*handles[handle]->ploth_setbg)(handle,background));}

/* Set writing mode (replace/xor/or/erase/...) */
int plot_setmode(int handle, int mode){
  return((*handles[handle]->ploth_setmode)(handle,mode));}

/* Set data window */
void plot_setwin(int handle, double xmin, double ymin,
		             double xmax, double ymax){
   handles[handle]->xwmin = xmin;   handles[handle]->xwmax = xmax;
   handles[handle]->ywmin = ymin;   handles[handle]->ywmax = ymax;
   calculate_transformation(handle);
}

/* Set data viewport */
void plot_setview(int handle, int xmin, int ymin, int xmax, int ymax){
   handles[handle]->xvmin = xmin;   handles[handle]->xvmax = xmax;
   handles[handle]->yvmin = ymin;   handles[handle]->yvmax = ymax;
   calculate_transformation(handle);
}

/* set clipping */
void plot_setclip(int handle, int clip){
   handles[handle]->clip = clip;
}

/* set symbol size */
double plot_setsymsize(int handle, double size){
   handles[handle]->symbol_factor = size;
   return(handles[handle]->symbol_factor);
}


/*====================== inquire Parameters ====================*/
/* get foreground colour index */
int plot_getfg(int handle){
   return((*handles[handle]->ploth_getfg)(handle));
}


/* get background colour index */
int plot_getbg(int handle){
   return((*handles[handle]->ploth_getbg)(handle));
}


/* get writing mode (replace/xor/or/erase/...) */
int plot_getmode(int handle){
   return((*handles[handle]->ploth_getmode)(handle));
}


/* get data window */
void plot_getwin(int handle, double *xmin, double *xmax,
		             double *ymin, double *ymax){
    *xmin = handles[handle]->xwmin;    *xmax = handles[handle]->xwmax;
    *ymin = handles[handle]->ywmin;    *ymax = handles[handle]->ywmax;
}

/* get data viewport */
void plot_getview(int handle, int *xmin, int *xmax, int *ymin, int *ymax){
    *xmin = handles[handle]->xvmin;    *xmax = handles[handle]->xvmax;
    *ymin = handles[handle]->yvmin;    *ymax = handles[handle]->yvmax;
}


/* get clipping */
int plot_getclip(int handle){return(handles[handle]->clip);
}


/* get symbol size */
double plot_getsymsize(int handle){return(handles[handle]->symbol_factor);
}


/*=========== Output routines, world coordinates only ============*/
int in_range(int handle, double x, double y)
{ if (!handles[handle]->clip) return(1);
  if (x > handles[handle]->xwmax) return(0);
  if (y > handles[handle]->ywmax) return(0);
  if (x < handles[handle]->xwmin) return(0);
  if (y < handles[handle]->ywmin) return(0);
  return(1);
}

void plot_point(int handle, double X, double Y){
   int IX, IY;
   if (in_range(handle,X,Y)){
      world_to_screen(handle,X,Y,&IX,&IY);
      (*handles[handle]->ploth_point)(handle,X,Y);
}}

void plot_line(int handle, double X0, double Y0, double X1, double Y1){
   int IX0, IY0, IX1, IY1;
   if ((in_range(handle,X0,Y0)) && (in_range(handle,X1,Y1))){
      world_to_screen(handle,X0,Y0,&IX0,&IY0);
      world_to_screen(handle,X1,Y1,&IX1,&IY1);
      (*handles[handle]->ploth_line)(handle,IX0,IY0,IX1,IY1);
}}


/*=================== Text output ================*/
/* set text alignment */
void plot_settextalign(int handle, int xalign, int yalign){
   (*handles[handle]->ploth_settextalign)(handle,xalign,yalign);
}

/* write txt at position (X,Y) */
void plot_writetext(int handle, int worldcoor, double X, double Y, char txt[]){
   int IX,IY;
   if (worldcoor) world_to_screen(handle,X,Y,&IX,&IY);
   else { IX = X; IY = Y; }
   (*handles[handle]->ploth_writetext)(handle,IX,IY,txt);
}

/* erase txt at position (X,Y). Usually done by plot_setmode(ERASE) and */
/* plot_writetext, but in some cases it has to be done in another way...*/
void plot_erasetext(int handle, int worldcoor, double X, double Y, char txt[]){
   int IX,IY;
   if (worldcoor) world_to_screen(handle,X,Y,&IX,&IY);
   else { IX = X; IY = Y; }
   (*handles[handle]->ploth_erasetext)(handle,IX,IY,txt);
}


/*================= Pointer position ==============*/

/* Get pointer position and return terminator. Returns 0 if handle if */
/* output only (Plotter,...), character if a key was pressedm, or one */
/* of the codes defined as PLOT_TRM_... in "u_pl_def.h"               */
int plot_getpointer(double *X, double *Y){
   int IX, IY, trm, handle;
   trm =plotd_getpointer_X11(&handle,mhandle,&IX,&IY);
   a_h=handle;
  if (trm != PLOT_TRM_ABORT) screen_to_world(a_h,IX,IY,X,Y);
   return(trm);
}

/* Check for event, if event present, get pointer position and return */
/* terminator. Returns 0 if handle if output only (Plotter,...) or    */
/* no event, character if a key was pressedm, or one of the codes     */
/* defined as PLOT_TRM_... in "u_pl_def.h"                            */
int plot_checkevent(int handle, double *X, double *Y){
   int IX, IY, trm;
   if (handles[handle]->ploth_checkevent==NULL) return 0;
   trm = (*handles[handle]->ploth_checkevent)(handle,&IX,&IY);

   if (trm != PLOT_TRM_ABORT) screen_to_world(handle,IX,IY,X,Y);
   return(trm);
}

/*==================== Future work ================*/
/* Set font for text in graphic mode */
/* Define color                      */


void plot_lineabs(int handle, double x1, double y1, double x2, double y2){
   plot_line(handle,x1,y1,x2,y2);
   handles[handle]->akt_x = x2; handles[handle]->akt_y = y2;
}

void plot_rectangle(int handle, double x1, double y1, double x2, double y2,
                    int filled){
   if (handles[handle]->ploth_rectangle==NULL){
      plot_lineabs(handle,x1,y1,x2,y1); plot_lineto(handle,x2,y2);
      plot_lineto(handle,x1,y2); plot_lineto(handle,x1,y1);
   } else 
     handles[handle]->ploth_rectangle(handle, x1, y1, x2, y2, filled);
}

void plot_lineto(int handle, double x, double y){
   plot_lineabs(handle,handles[handle]->akt_x,
		       handles[handle]->akt_y,x,y);
}

void plot_linerel(int handle, double x, double y){
   x = handles[handle]->akt_x + x; y = handles[handle]->akt_y + y;
   plot_lineto(handle,x,y);
}

void plot_moverel(int handle, double x, double y){
   handles[handle]->akt_x = handles[handle]->akt_x + x;
   handles[handle]->akt_y = handles[handle]->akt_y + y;
}

void plot_moveto(int handle, double x, double y){
   handles[handle]->akt_x = x;
   handles[handle]->akt_y = y;
}

void plot_polyline(int handle, int n, double x[], double y[]){
   int *ix, *iy, i, j, size;
   size = n*sizeof(int); ix = malloc(size); iy = malloc(size);
   if ((ix==NULL) || (iy==NULL)){
      free(ix); free(iy);
      status = PLOT_STAT_NOMEMORY; return;}
   j = 0;
   for (i=0; i<n; i++) {
      if (in_range(handle,x[i],y[i])){
        world_to_screen(handle,x[i],y[i],ix+j,iy+j); j++;
      } else {if (j>1) (*handles[handle]->ploth_polyline)(handle,j,ix,iy);j=0;}
   }
   if (j>1) (*handles[handle]->ploth_polyline)(handle,j,ix,iy);
   free(ix); free(iy);
}



void plot_symbol(int handle, int sym, double x, double y)
{  int i, ix, iy;
   if (! in_range(handle,x,y)) return;
   world_to_screen(handle,x,y, &ix, &iy);
   if ((symbols_loaded) && (sym>0) && (sym<34)){
     for (i=0; i<symbols[sym].vec; i++)
	(*handles[handle]->ploth_line)(handle,
	   (int)(ix+symbols[sym].x1[i] * handles[handle]->symbol_factor),
	   (int)(iy+symbols[sym].y1[i] * handles[handle]->symbol_factor),
	   (int)(ix+symbols[sym].x2[i] * handles[handle]->symbol_factor),
	   (int)(iy+symbols[sym].y2[i] * handles[handle]->symbol_factor));
   } else (*handles[handle]->ploth_point)(handle,ix,iy);
}

void plot_polysymbol(int handle, int pts, int sym, double X[], double Y[]){
   int i, *ix, *iy, ipts;
   if ((symbols_loaded) && (sym>0) && (sym<34)){
     if (handles[handle]->ploth_polysymbol==NULL)
       for (i=0; i<pts; i++) plot_symbol(handle,sym,X[i],Y[i]);
     else{
       ix = malloc(pts * sizeof(int));
       iy = malloc(pts * sizeof(int));
       ipts = 0;
       for (i=0; i<pts; i++) {
         if (! in_range(handle,X[i],Y[i])) continue;
         world_to_screen(handle,X[i],Y[i], &ix[ipts], &iy[ipts]);
	 ipts++;
       }
       (*handles[handle]->ploth_polysymbol)(handle,ipts,symbols[sym],ix,iy);
       free(ix); free(iy);
   }} else for (i=0; i<pts; i++) plot_point(handle,X[i],Y[i]);
}


static void mos_scaling(int nda,int nth, 
                        double *xwmin, double *xwmax,
                        double *ywmin, double *ywmax){
   int i;
   if(nda>0){if(plot.xd[0]<plot.xd[nda-1]){
               *xwmin = plot.xd[0];*xwmax = plot.xd[nda-1];
             }else{
               *xwmax = plot.xd[0];*xwmin = plot.xd[nda-1];}
   }
   if(nth>0){if(plot.xt[0]<plot.xt[nth-1]){
               *xwmin = plot.xt[0];*xwmax = plot.xt[nth-1];
             }else{
               *xwmax = plot.xt[0];*xwmin = plot.xt[nth-1];}
   }

   /*printf("*xwmin= %f *xwmax= %f \n",*xwmin,*xwmax);*/
   *ywmin = VHIGH;  
   *ywmax = -VHIGH; 
   
    if (nda >0) {
     for (i=0; i<nda; i++){
       if (plc[a_h].errorbars){
        if (*ywmax < plot.yd[i]+plot.yderr[i]) 
            *ywmax = plot.yd[i]+plot.yderr[i];
        if (*ywmin > plot.yd[i]-plot.yderr[i]) 
            *ywmin = plot.yd[i]-plot.yderr[i];
        if(plot.nchanplot[3] > 1){
         if (*ywmax < plot.yres[i]+plot.yderr[i]) 
             *ywmax = plot.yres[i]+plot.yderr[i];
         if (*ywmin > plot.yres[i]-plot.yderr[i])  
             *ywmin = plot.yres[i]-plot.yderr[i];
	}
       }
       else{
        if (*ywmax < plot.yd[i]) *ywmax = plot.yd[i];
        if (*ywmin > plot.yd[i]) *ywmin = plot.yd[i];
        if(plot.nchanplot[3] >1){
         if (*ywmax < plot.yres[i])  *ywmax = plot.yres[i];
         if (*ywmin > plot.yres[i])  *ywmin = plot.yres[i];
	}
       }
      }
     }
     if (have_theo){
      for (i=0; i<nth; i++){
       if (*ywmin > plot.yt[i]) *ywmin = plot.yt[i];
       if (*ywmax < plot.yt[i]) *ywmax = plot.yt[i];
      }
     }
   }

static void sub_scaling(int nth, 
                        double *xwmin, double *xwmax,
                        double *ywmin, double *ywmax){
   int i,npts;
   /*printf("sub_scaling n_sub=%i  nth=%i\n",n_sub,nth);*/
     if (n_sub >0){
      npts=n_sub*nth;
      for (i=0; i<npts; i++){
       if (*ywmin > plot.plotsub[i]) *ywmin = plot.plotsub[i];
       if (*ywmax < plot.plotsub[i]) *ywmax = plot.plotsub[i];
      }
     }
   }

static void init_plcfg(){
   int i;
   plc[a_h].autoaxis  = TRUE;
   plc[a_h].scaling   = SC_EXACT;
   plc[a_h].errorbars = FALSE;
   for (i=0; i< MAXBUF; i++) plc[a_h].selected[i] = FALSE;
   plc[a_h].title[0] = 0;
   plc[a_h].xlabel[0] = 0; strcpy(plc[a_h].xformat,DEFAULTFORMATX);
   plc[a_h].ylabel[0] = 0; strcpy(plc[a_h].yformat,DEFAULTFORMATY);
   plc[a_h].xmin = 0; plc[a_h].xmax = 1; plc[a_h].xstart = 0; plc[a_h].xstep = 0;
   plc[a_h].ymin = 0; plc[a_h].ymax = 1; plc[a_h].ystart = 0; plc[a_h].ystep = 0;
   plc[a_h].xtics = 5; plc[a_h].ytics = 5;
}

void init_plot(int handle,char plotdev[]){
   int devtype, wait;

   init_v_inp();
   if (plotdev[0]=='\0'){ strcpy(plotdev,"default"); }
   wait = FALSE;
   if (! plot_init()){
      printf("Error loading Symbolfile: no symbols in Plot");
      wait = TRUE;   }
   devtype = PLOT_DEV_X11;
   if (devtype == PLOT_DEV_NO){
      a_h = -1;
      wait = TRUE;
   }else{
      
  /*    printf("1plot_open  winposx=%i  winposy=%i\n",winposx,winposy);*/
      a_h = plot_open(handle,devtype,winsizex,winsizey,
                winposx,winposy,wdescription);
      if (a_h < 0){
         printf(" Error initializing Plot\n %s\n",plot_message());
         wait = TRUE;
      }else{
	 init_plcfg();
	 is_window_device = 
         plot_capabilities(a_h,PLOT_CAPABILITY_WINDOWS);
      }

   }
   taste_warten(wait);
}

static void round_scale(double *u,double *o) {
   double fact, diff, oo, uu;
   oo = *o; uu = *u;
   diff = oo - uu;
   fact = 1;
   while (diff < 10)  { diff = diff*10; fact = fact/10;}
   while (diff > 100) { diff = diff/10; fact = fact*10;}
   *u = floor( uu / fact ) * fact;
   *o = ceil( oo / fact ) * fact;
   while( *o <= oo) *o = *o + fact;
   while( *u >= uu) *u = *u - fact;
}

void do_scaling(Scaling scale){
   int i;
   double tmp;
   if (scale == SC_AXIS){
      if ((plc[a_h].xstep <= 0.0) || (plc[a_h].ystep <= 0.0)) return;
      if (plc[a_h].xmax < plc[a_h].xstart) return;
      if (plc[a_h].ymax < plc[a_h].ystart) return;
      if (((plc[a_h].xmax - plc[a_h].xstart)/plc[a_h].xstep) > 20) return;
      if (((plc[a_h].ymax - plc[a_h].ystart)/plc[a_h].ystep) > 20) return;
      plc[a_h].autoaxis = FALSE;
      plc[a_h].scaling = SC_OFF;

      plc[a_h].xmin = plc[a_h].xstart; plc[a_h].ymin = plc[a_h].ystart;
      tmp = plc[a_h].xmin;
      do { tmp = tmp + plc[a_h].xstep; } while(tmp < plc[a_h].xmax);
      plc[a_h].xmax = tmp;
      tmp = plc[a_h].ymin;
      do { tmp = tmp + plc[a_h].ystep; } while(tmp < plc[a_h].ymax);
      plc[a_h].ymax = tmp;
      return;
   }

   mos_scaling(n_data,n_theo,
    &plc[a_h].xmin,&plc[a_h].xmax,&plc[a_h].ymin,&plc[a_h].ymax);
   if (scale == SC_ROUND){
      round_scale(&plc[a_h].xmin,&plc[a_h].xmax);
      round_scale(&plc[a_h].ymin,&plc[a_h].ymax);
}}


static int check_range(){
  if ((plc[a_h].xmin < plc[a_h].xmax) && (plc[a_h].xmin < plc[a_h].xmax)) return TRUE;
  printf(" Error: Illegal range\n"); return FALSE;
}

static int ask_range()
{  char ch, chh;
   static char scale_state[3][6] = {"EXACT","ROUND","OFF"};
   static char off_on[2][4] = {"OFF","ON"};
  return TRUE;
}


static void write_number(int handle, int x, int y, double v,
			 int hor, int ver, char format[])
{ char outstr[80];

  plot_settextalign(handle,hor,ver);
  sprintf(outstr,format,v);
  plot_writetext(handle,TRUE,x,y,outstr);
}

static void simple_axis(int handle,int x1, int y1, int x2, int y2){
   int i,xi[TICS],yi[TICS];
  for (i=0;i<TICS;i++){
     xi[i] = i /(TICS-1.0) *(x2-x1) + x1;
     yi[i] = i /(TICS-1.0) *(y2-y1) + y1;
  }
  for (i=0;i<TICS;i++)
     plot_line(handle,xi[i],y1-frame,xi[i],y1-frame+ticlen);
  for (i=0;i<TICS;i++)
     plot_line(handle,xi[i],y2+frame-ticlen,xi[i],y2+frame);
  for (i=0;i<TICS;i++)
     plot_line(handle,x1-frame,yi[i],x1-frame+ticlen,yi[i]);
  for (i=0;i<TICS;i++)
     plot_line(handle,x2+frame-ticlen,yi[i],x2+frame,yi[i]);
   write_number(handle,sizex*LEFT_FRAME-3*frame,
                       sizey*BOTTOM_FRAME-3*frame,plc[a_h].xmin,
                PLOT_LEFTTEXT,PLOT_TOPTEXT,plc[a_h].xformat);
   write_number(handle,sizex*RIGHT_FRAME,
		       sizey*BOTTOM_FRAME-3*frame,plc[a_h].xmax,
                PLOT_RIGHTTEXT,PLOT_TOPTEXT,plc[a_h].xformat);
   write_number(handle,sizex*LEFT_FRAME-3*frame,
		       sizey*BOTTOM_FRAME,plc[a_h].ymin,
                PLOT_RIGHTTEXT,PLOT_CENTERTEXT,plc[a_h].yformat);
   write_number(handle,sizex*LEFT_FRAME-3*frame,
		       sizey*TOP_FRAME,plc[a_h].ymax,
                PLOT_RIGHTTEXT,PLOT_CENTERTEXT,plc[a_h].yformat);
}

static void draw_tics(int handle, int kk, int x_or_y, int is_long, double val,
		      int pos1, int pos2){
   int len;
   if (is_long) len = 2*ticlen; else len = ticlen;
   if (x_or_y){   /* Y-Axis */
     plot_line(handle,pos1-frame-len,kk,pos1-frame+len,kk);
     plot_line(handle,pos2+frame-len,kk,pos2+frame+len,kk);
     if (is_long) write_number(handle,sizex*LEFT_FRAME-3*frame,kk,val,
                PLOT_RIGHTTEXT,PLOT_CENTERTEXT,plc[a_h].yformat);
   }else{         /* X-Axis */
     plot_line(handle,kk,pos1-frame-len,kk,pos1-frame+len);
     plot_line(handle,kk,pos2+frame-len,kk,pos2+frame+len);
     if (is_long) write_number(handle,kk+FRAME,sizey*BOTTOM_FRAME-3*frame,val,
                PLOT_RIGHTTEXT,PLOT_TOPTEXT,plc[a_h].xformat);
   }
}


static void customized_axis(handle,x1,y1,x2,y2){
   int xk, yk, i, istepx, istepy, istartx, istarty,
       ixnull, iynull;
   double val, ticx, ticy;
   plot_setview(handle,sizex*LEFT_FRAME,sizey*BOTTOM_FRAME,
                sizex*RIGHT_FRAME, sizey*TOP_FRAME);
   plot_setwin(handle,plc[a_h].xmin,plc[a_h].ymin,plc[a_h].xmax,plc[a_h].ymax);
   world_to_screen(handle,0.0,0.0,&ixnull,&iynull);
   world_to_screen(handle,plc[a_h].xstart,plc[a_h].ystart,&istartx,&istarty);
   world_to_screen(handle,plc[a_h].xstep,plc[a_h].ystep,&istepx,&istepy);
   istepx = istepx - ixnull; istepy = istepy - iynull;
   ticx = (double)istepx / plc[a_h].xtics;
   ticy = (double)istepy / plc[a_h].ytics;

   plot_setwin(handle,0,0,sizex,sizey);
   plot_setview(handle,0,0,sizex,sizey);
   val = plc[a_h].xstart;
   for (xk=istartx; xk<=x2+FRAME; xk = xk+istepx){
     draw_tics(handle,xk,0,1,val,y1,y2);
     for (i=1; i<plc[a_h].xtics; i++)
       if (xk+ticx*i < x2)
         draw_tics(handle,(int)(xk+ticx*i),0,0,0.0,y1,y2);
     val = val + plc[a_h].xstep;
   }
   val = plc[a_h].ystart;
   for (yk=istarty; yk<=y2+FRAME; yk = yk+istepy){
     draw_tics(handle,yk,1,1,val,x1,x2);
     for (i=1; i<plc[a_h].ytics; i++)
       if (yk+ticy*i < y2)
         draw_tics(handle,(int)(yk+ticy*i),1,0,0.0,x1,x2);
     val = val + plc[a_h].ystep;
   }
}


static int check_axis(){
   if (plc[a_h].xtics < 1) plc[a_h].xtics = 1;
   if (plc[a_h].ytics < 1) plc[a_h].ytics = 1;

   if ((plc[a_h].xstep <= 0.0) || (plc[a_h].ystep <= 0.0)) return FALSE;

   if (plc[a_h].xmin > plc[a_h].xstart) return FALSE;
   if (plc[a_h].xmax < (plc[a_h].xstart+plc[a_h].xstep)) return FALSE;
   if (((plc[a_h].xmax - plc[a_h].xmin)/plc[a_h].xstep) > 20) return FALSE;

   if (plc[a_h].ymin > plc[a_h].ystart) return FALSE;
   if (plc[a_h].ymax < (plc[a_h].ystart+plc[a_h].ystep)) return FALSE;
   if (((plc[a_h].ymax - plc[a_h].ymin)/plc[a_h].ystep) > 20) return FALSE;

   return TRUE;
}


static void draw_frame(int handle,int x1, int y1, int x2, int y2)
{ int use_the_axis;
  plot_setfg(handle,PLOT_USEDEFAULT);
  if (! plc[a_h].autoaxis){
     use_the_axis = check_axis();
     if (! use_the_axis)
        printf(" Warning: Customized axis could not be used\n");
  } else use_the_axis = FALSE;
  plot_line(handle,x1-frame,y1-frame,x2+frame,y1-frame);
  plot_line(handle,x1-frame,y2+frame,x2+frame,y2+frame);
  plot_line(handle,x1-frame,y1-frame,x1-frame,y2+frame);
  plot_line(handle,x2+frame,y1-frame,x2+frame,y2+frame);
  if (use_the_axis) customized_axis(handle,x1,y1,x2,y2);
               else simple_axis(handle,x1,y1,x2,y2);
}

/*   
** xx, yy (double) are out parameters calculated from xin,yin (float)
*/ 

static void one_curve_data(int handle, int n_pts,
                      double x[], double y[], double err[], int col){
   int i;
   plot_setfg(handle,col);
       
/*      for (i=0; i<20; i++){
   printf("  i   %i ",i);
   printf(" x %f",x[i]);
   printf(" y %f\n",y[i]);}*/

       plot_polysymbol(handle,n_pts,1,x,y);
      if (plc[a_h].errorbars){ 
       for (i=0; i<n_pts; i++)
       plot_line(handle,x[i],y[i]-err[i],x[i],y[i]+err[i]);
      }
}

static void one_curve_theory(int handle, int n_pts,
                      double x[], double y[], int col){
   plot_setfg(handle,col);
   plot_polyline(handle,n_pts,x,y); /* Theory or Subspectrum */
}


static void mos_plot(int handle){
   int k, i, col;
   double p_sub[MAXN];
   if (plot_data){
     one_curve_data(handle,n_data,plot.xd,plot.yd,plot.yderr,PLOT_USEDEFAULT);
     if(plot.nchanplot[3] > 1){
       /*  printf(" plot.nchanplot[3]  %i \n",plot.nchanplot[4]);*/
     for ( i=0; i<n_data; i++ ){p_sub[i]=plot.yres[i];}
     one_curve_data(handle,n_data,plot.xd,p_sub,plot.yderr,PLOT_USEDEFAULT);
     }}       
   if (have_theo){
      n_sub=plot.n_subspectra;
/*   printf(" +++++++++++++++++++++++theo n_sub  %i \n",n_sub);*/
      /*   printf(" have_theo %i\n",have_theo);
	   printf(" plot_theo %i\n",plot_theo);*/

      if (plot_theo) one_curve_theory(handle,n_theo,plot.xt,plot.yt,PLOT_USEDEFAULT);
      if (plot_subspectra){
      if (n_sub > 0) {
        for (k=0; k<n_sub; k++){
         col = ((k+2) % 7) + 2;
         n_pts=k*n_theo;
	 if(pl_subsp[k]){
          for ( i=0; i<n_theo; i++ ){p_sub[i]=plot.plotsub[n_pts+i];}
          one_curve_theory(handle,n_theo,plot.xt,p_sub,col);
	 }
   }}}}
}

static void do_the_plot(int handle){
   int i;
   plot_getdev(handle,&winposx,&winposy,&sizex,&sizey);

/* plot_getdev returns wrong values*/
   winposx=winposx-2;
   winposy=winposy-24;
/* plot_getdev returns wrong values*/

   size_factor = sizex / 1000;
   if (size_factor < 1.0) size_factor = 1.0;
   /*printf("handle=%d size_factor=%10.3f \n",handle,size_factor);*/

   plot_setsymsize(handle,size_factor);
   frame = FRAME * size_factor;
   ticlen = TICLEN * size_factor;
   plot_clear(handle);
   plot_setmode(handle,PLOT_MODE_OVER);
   plot_setwin(handle,0,0,sizex,sizey);
   plot_setview(handle,0,0,sizex,sizey);
   plotd_sel_font(handle,sizey);

   draw_frame(handle,sizex*LEFT_FRAME,sizey*BOTTOM_FRAME,
	      sizex*RIGHT_FRAME, sizey*TOP_FRAME);
   plot_settextalign(handle, PLOT_LEFTTEXT,PLOT_BOTTOMTEXT); 
   plot_writetext(handle, FALSE,
	      sizex*LEFT_FRAME,
	      sizey*TITLE_POSY,plc[a_h].title);
   plot_settextalign(handle, PLOT_RIGHTTEXT,PLOT_BOTTOMTEXT); 
   plot_writetext(handle, FALSE,
	      sizex*RIGHT_FRAME,
	      sizey*XLABEL_POSY,plc[a_h].xlabel);
   plot_settextalign(handle, PLOT_LEFTTEXT,PLOT_BOTTOMTEXT); 
   plot_writetext(handle, FALSE, FRAME,
	      sizey*YLABEL_POSY,plc[a_h].ylabel);
   /*printf("do_the_plot xlabel=%s\n",plc[a_h].xlabel);*/
   /*   if(plc[a_h].y2label[0] != 0){*/
   plot_writetext(handle, FALSE, FRAME,
		    sizey*Y2LABEL_POSY,plc[a_h].y2label);

   plot_setview(handle,sizex*LEFT_FRAME,sizey*BOTTOM_FRAME,
                sizex*RIGHT_FRAME, sizey*TOP_FRAME);
    plot_setwin(handle,plc[a_h].xmin,plc[a_h].ymin,plc[a_h].xmax,plc[a_h].ymax);
   plot_setclip(handle,TRUE);
   mos_plot(handle);
}

void updatetst(){
   int save_plotdata;
   save_plotdata = plot_data;
   a_h=win.whandle[win.ksphandle-1];
   if (! do_auto_update) return;
   if (a_h == -1) return;
   if (! is_window_device) return;
   plot_data = FALSE; 
   plot_subspectra = TRUE;
   plot_theo = TRUE;
   have_theo = TRUE;
   n_theo = plot.nchanplot[4];
   mos_plot(a_h);
   plot_data = save_plotdata;
}

void updatefit(){
   int save_plotsub,save_plotdata;
   a_h=win.whandle[win.ksphandle-1];
   if (! do_auto_update) return;
   if (a_h == -1) return;
   if (! is_window_device) return;
   save_plotsub = plot_subspectra;
   save_plotdata = plot_data;
   plot_subspectra = FALSE;
   plot_data = FALSE; 
   plot_theo = TRUE;
   have_theo = TRUE;
   n_theo = plot.nchanplot[4];
   mos_plot(a_h);
   plot_subspectra = save_plotsub;
   plot_data = save_plotdata;
}

static void write_info(char info[]){
    static char savetext[80] = "";
    int posx, posy;
    plot_settextalign(a_h,PLOT_CENTERTEXT,PLOT_BOTTOMTEXT); 
    posx = sizex*INFO_POSX;    posy = sizey*TOP_FRAME+1.5*frame;
    if (savetext[0]!=0) plot_erasetext(a_h,FALSE,posx, posy, savetext);
    plot_writetext(a_h,FALSE,posx,posy,info);
    strcpy(savetext,info);
}


static void help(){
   plot_textmode(); clrscr();
   printf("     Plot commands (upper/lower case)\n");
   printf("     ================================\n");
   printf("  *,Q          - Quit\n");
   printf("  H            - Help: this page\n");
   printf("  ^W,W         - Replot\n");
   printf("  R            - Rescale\n");
   printf("  ^R           - Replot\n");
   printf("  E            - Errorbars (toggle)\n");
   printf("  Z            - Zoom\n");
   printf("  U            - UnZoom\n");
   printf("  S            - Subspectra (toggle)\n");
   printf("  1,2,..       - Subspectrum 1,2,...(toggle)\n");
   printf("  D            - Data (toggle)\n");
   printf("  T            - Theory (toggle)\n");
   printf("  A            - Hold scale (toggle)\n");
   printf("  Any other key: show cursor Position\n");
   taste_warten(TRUE);
}


static void zoom(){
   double tmp;
   int ix, iy;
#define exchange(x,y) tmp=x; x=y; y=tmp;
   plc[a_h].last_xmin = plc[a_h].xmin;   plc[a_h].last_xmax = plc[a_h].xmax;
   plc[a_h].last_ymin = plc[a_h].ymin;   plc[a_h].last_ymax = plc[a_h].ymax;
   write_info("Mark first corner");
   plot_getpointer(&plc[a_h].xmin,&plc[a_h].ymin);
   write_info("Mark second corner");
   plot_getpointer(&plc[a_h].xmax,&plc[a_h].ymax);
   if (plc[a_h].ymax < plc[a_h].ymin) {exchange(plc[a_h].ymax,plc[a_h].ymin);}
   if (plc[a_h].xmax < plc[a_h].xmin) {exchange(plc[a_h].xmax,plc[a_h].xmin);}
   plc[a_h].zoomed = TRUE;
#undef exchange
}

/* Swap actual and last data window */
static void unzoom(){
   double tmp;
#define exchange(x,y) tmp=x; x=y; y=tmp;
   if (! plc[a_h].zoomed) return;
   exchange(plc[a_h].xmin, plc[a_h].last_xmin);
   exchange(plc[a_h].xmax, plc[a_h].last_xmax);
   exchange(plc[a_h].ymin, plc[a_h].last_ymin);
   exchange(plc[a_h].ymax, plc[a_h].last_ymax);
#undef exchange
}


static void rescale(){
   plc[a_h].last_xmin = plc[a_h].xmin;   plc[a_h].last_xmax = plc[a_h].xmax;
   plc[a_h].last_ymin = plc[a_h].ymin;   plc[a_h].last_ymax = plc[a_h].ymax;
   if (plc[a_h].scaling == SC_OFF) do_scaling(SC_EXACT);
   else  do_scaling(plc[a_h].scaling);
}

static int is_graphmode = FALSE;

int plot_the_buffers_noask(int k){
  double x, y;
  int trm ,lhandle;
  char out[40];
  int lfirst; 

  lfirst=0; 
  do{
    lfirst=lfirst+1;
    lhandle=a_h;
    if (!is_graphmode) plot_graphmode();
    is_graphmode = TRUE;
    
    do_the_plot(a_h);
    if(lfirst == 1) plotd_focus_X11(a_h);
    plot_setfg(a_h,PLOT_USEDEFAULT);
    if (plot_auto){plot_auto=FALSE;  return '*';}
    write_info("press H to get help");
    if(k==FALSE) return 'M';
    do{ 
      trm = plot_getpointer(&x,&y);
      /*printf("plot_the_buffers_noask x=%10.2f y=%10.2f \n",x,y);*/
      /*printf("plot_the_buffers_noask lhandle=%d a_h=%d \n",lhandle,a_h);*/
      if(lhandle!=a_h) return 'C';
      trm = toupper(trm);
      if (trm==PLOT_TRM_MB_RIGHT) return '*';
      if (trm=='Q') return '*';
      if (trm=='*') return '*';
      if (trm=='W') break;
      if (trm==23)  break;  /* Ctrl-W */
      if (trm==PLOT_TRM_REPLOT) break;
      if (trm==PLOT_TRM_ALREADY_ACTIVE) return trm;
      if (trm=='R') {rescale(); break;}
      if (trm==18) {sub_scaling(n_theo,
                    &plc[a_h].xmin,&plc[a_h].xmax,&plc[a_h].ymin,&plc[a_h].ymax);
                    break;} /* Ctrl-R */
      if (trm=='H') {help();  is_graphmode=FALSE; break;}
      if (trm=='Z') {zoom(); break;}
      if (trm=='D') {plot_data = (! plot_data); break;}
      if (trm=='T') {plot_theo = (! plot_theo); break;}
      if (trm=='E') {plc[a_h].errorbars= (! plc[a_h].errorbars); break;}
      if (trm=='|') {plc[a_h].errorbars= TRUE; break;}
      if (trm=='\\') {plc[a_h].errorbars= FALSE; break;}
      if (trm=='A') {if(plc[a_h].scaling==SC_OFF){plc[a_h].scaling=SC_EXACT;}
                     else{plc[a_h].scaling=SC_OFF;};break;}
      if (trm=='S'){ plot_subspectra=(! plot_subspectra);
                     is_graphmode=FALSE; break;}
      if (trm=='1'){ pl_subsp[0]=(! pl_subsp[0]);is_graphmode=FALSE; break;}
      if (trm=='2'){ pl_subsp[1]=(! pl_subsp[1]);is_graphmode=FALSE; break;}
      if (trm=='3'){ pl_subsp[2]=(! pl_subsp[2]);is_graphmode=FALSE; break;}
      if (trm=='4'){ pl_subsp[3]=(! pl_subsp[3]);is_graphmode=FALSE; break;}
      if (trm=='5'){ pl_subsp[4]=(! pl_subsp[4]);is_graphmode=FALSE; break;}
      if (trm=='6'){ pl_subsp[5]=(! pl_subsp[5]);is_graphmode=FALSE; break;}
      if (trm=='7'){ pl_subsp[6]=(! pl_subsp[6]);is_graphmode=FALSE; break;}
      if (trm=='8'){ pl_subsp[7]=(! pl_subsp[7]);is_graphmode=FALSE; break;}

      if (trm=='U') {unzoom(); break;}
      sprintf(out,"%10.3f  %15.3f",x,y);
      if (is_window_device) printf("%s\n",out);
      write_info(out);
    }while(k);

  }while((trm!='*') && (trm!='M'));
  return trm;
}

static void do_plot_the_buffers(int k)
{
   int trm;
   if (a_h < 0) return;
   if (plc[a_h].scaling != SC_OFF) do_scaling(plc[a_h].scaling);
   if (plc[a_h].xmin >= plc[a_h].xmax) {
     printf("** error**  xmosplo:  xmin >= xmax \n");
     win.ksphandle=-2;
     return;}
   plc[a_h].zoomed = FALSE;
   do{
     if (!ask_range()) return;
     trm = plot_the_buffers_noask(k);
     if (trm==PLOT_TRM_ALREADY_ACTIVE) return;
     is_graphmode = FALSE;
     plot_textmode();
     clrscr();
       /*      printf("do_plot_the_buffers() trm  %d M %d\n",trm,'M');*/
     win.posx[win.ksphandle-1]=winposx;    
     win.posy[win.ksphandle-1]=winposy;
     win.sizex[win.ksphandle-1]=sizex;
     win.sizey[win.ksphandle-1]=sizey;
/*     printf("ksphandle=%i  sizex=%i sizey=%i winposx=%i  winposy=%i\n",\
       win.ksphandle,sizex,sizey,winposx,winposy);*/
     if(trm=='*') win.ksphandle=-1;
     if(trm=='M'){ win.ksphandle=0;trm='*';}
     if(trm=='C'){win.ksphandle=wksp[a_h]+1;trm='*';}
   }while(trm!='*');
}


static void copy_title(){
   char *tmp;
   int i, ii;
   tmp = win.wdescript;
   for (i=0; i<79; i++){
       ii = i + (i/2)*2;
       plc[a_h].wdescript[i] = tmp[i]; 
   };plc[a_h].wdescript[79] = 0;
   tmp = plot.title;
   for (i=0; i<79; i++){
       ii = i + (i/2)*2;
       plc[a_h].title[i] = tmp[i]; 
   };plc[a_h].title[79] = 0;
   tmp= plot.axes[0];
   for (i=0; i<24; i++){if(tmp[i]==' '){plc[a_h].xlabel[i] = 0;}
   else{plc[a_h].xlabel[i] = tmp[i];}}
   tmp= plot.axisformat[0];
   for (i=0; i<16; i++){if(tmp[i]==' '){plc[a_h].xformat[i] = 0;}
   else{plc[a_h].xformat[i] = tmp[i];}}
   plc[a_h].xformat[16]=0;

   tmp= plot.axisformat[0];
   for (i=0; i<16; i++){if(tmp[i+16]==' '){plc[a_h].yformat[i] = 0;}
   else{plc[a_h].yformat[i] = tmp[i+16];}}
   plc[a_h].yformat[16]=0;

   plc[a_h].xlabel[23] = 0;
   plc[a_h].y2label[0] = 0;
   plc[a_h].ylabel[23] = 0;
   plc[a_h].y2label[23] = 0;
   ii=0;
   tmp= plot.axes[0];
   for (i=0; i<24; i++){
     if(tmp[i+24] == ' ') {plc[a_h].ylabel[i] =0;ii=i+1;}
     else{ if(ii ==0){ plc[a_h].ylabel[i] = tmp[i+24];}
           else{ plc[a_h].y2label[i-ii] = tmp[i+24];}
     }
   }
}

void xmosplo(int *n1, int *iauto){
  
  int handle,i,ii,k;
  char *tmp;
     winsizex=win.sizex[win.ksphandle-1];
     winsizey=win.sizey[win.ksphandle-1];
     winposx=win.posx[win.ksphandle-1];
     winposy=win.posy[win.ksphandle-1];
/*     printf(" handle=%i winsizex=%i winsizey=%i winposx=%i  winposy=%i\n",\
       win.ksphandle,winsizex,winsizey,winposx,winposy);*/
     k=TRUE;
     if(win.hold == 0) k=FALSE;
   if (win.whandle[win.ksphandle-1]<0){
       handle=0;
       tmp = win.wdescript;
       for (i=0; i<79; i++){ii = i + (i/2)*2;
       wdescription[i] = tmp[i];};wdescription[79] = 0;
       init_plot(handle,"x");
       if(a_h <0) {win.ksphandle=-1; return;}
       do_auto_update = TRUE; 
       wptr.intptr[a_h]=(long)handles[a_h];
       win.whandle[win.ksphandle-1]=a_h;
   }else{   
    for (i=0; i<79; i++){wdescription[i] = plc[a_h].wdescript[i];}
   }

   a_h=win.whandle[win.ksphandle-1];
   wksp[a_h]=win.ksphandle-1;
   i=0;
   for(handle=0; handle<MAXHANDLES; handle++){ 
     if(win.whandle[handle]>=0){i=i+1;}
   }
   mhandle=i;

   copy_title();
   have_theo = (*n1 == 1);
   n_data = plot.nchanplot[0]; 
   n_theo = plot.nchanplot[4];
   n_sub =  plot.n_subspectra; 
   plot_subspectra = TRUE;
   for (i=0; i<n_sub; i++){pl_subsp[i]=TRUE;};
   plot_auto=(*iauto == 0);
   plot_data = TRUE; plot_theo = have_theo;
   do_plot_the_buffers(k);
}







