       subroutine allvec(n,ar,ai,wr,wi,vr,vi,vlr,vli,
     &                    fv1,fv2,fv3,n2,z,wm,ipvt)
C-----------------------------------------------------------
C
C    Find Right & Left eigenvalues/ vectors of matrix
C       (ar,ai)   - is NOT overwritten !
C        (wr,wi)  - eigenvalues
C        (vr,vi)  - Right Eigenvectors (by columns)
C        (vlr,vli)  - Left Eigenvectors (by rows)
C
C          (VLR+i*VLI) * (VR+i*VI) = I (Unit matrix)
C
C-----------------------------------------------------------
	implicit real*8 (a-h, o-z)
C   Parameters
C
C
C Working arrays
C
	real*8  ar(n,n),ai(n,n),wr(n),wi(n),
     &          vr(n,n),vi(n,n),vlr(n,n), vli(n,n),
     &          fv1(n),fv2(n),fv3(n),z(n2),wm(n2,n2)
        integer ipvt(n2),n,n2
	real*8  det(2),machep
C
 1000 format(16f8.3)
C
C       Copy initial Matrix to WM - for test purposes only
C
c      print *, 'AR', n
c      print 1000, ((ar(i,j), j = 1,n), i=1,n )
c      print *, 'AI', n
c      print 1000,((ai(i,j), j = 1,n), i=1,n )
CC
C     machep determines abort condition
c     
      machep=1.0d-6
      matz = 1
      CALL CG(n,ar,ai,wr,wi,matz,vr,vi,fv1,fv2,fv3,machep,ierr)
      if(ierr.ne.0) then
	    print *, 'CG failed while searching Eigenvectors'
	    stop
      endif
c      ar,ai,wr,wi,fv1,fv2,fv3 sind ok  
c      print *, 'VR', n
c      print 1000, ((vr(i,j), j = 1,n), i=1,n )
c      print *, 'VI', n
c      print 1000,((vi(i,j), j = 1,n), i=1,n )
c       print 1000,(fv1(i), i=1,n )
c       print 1000,(fv2(i), i=1,n )
c       print 1000,(fv3(i), i=1,n )
CCC   Normalize Eigenvectors
      do 2 i=1,n
  2   call stdvec(n,vr(1,i),vi(1,i))

C
C    Form Supermatrix
C
c      print *, 'VR', n
c      print 1000, ((vr(i,j), j = 1,n), i=1,n )
c      print *, 'VI', n
c      print 1000,((vi(i,j), j = 1,n), i=1,n )

	do 3 i=1,n
	do 3 j=1,n
	wm(i,j)=vr(i,j)
	wm(i+n,j+n)=vr(i,j)
	wm(i+n,j)=-vi(i,j)
	wm(i,j+n)=vi(i,j)
    3   continue
	job=01
C
C    Invert Matrix of Right Eigenvectors
C
	call dgeco(n2,wm,ipvt,rcond,z)
	if(rcond.lt.1.d-4) then
        	print *,'rcond=',rcond,' < 1.d-4'
		print *,'Bad conditioned matrix'
		stop
	endif
	call dgedi(n2,wm,ipvt,det,z,job)
	do 4 i=1,n
	do 4 j=1,n
         vlr(i,j)=wm(i,j)
         vli(i,j)=wm(i,j+n)
    4   continue
	return
	end

      subroutine stdvec(n,vr,vi)
C  'Standardize' complex vector
      implicit real*8 (a-h,o-z)
      dimension vr(n),vi(n)
      real*8 norm,h,rotr,roti

C  Find component with max imaginary part

      norm=0.d0
      im=1
      vimax=0.d0
      do 1 i=1,n
       norm=norm+vr(i)**2+vi(i)**2
       if(dabs(vi(i)).gt.vimax) then
	vimax=dabs(vi(i))
	im=i
       endif
    1 continue
      norm=dsqrt(norm)

      h=vr(im)**2+vi(im)**2
      if(h .gt. norm*1.d-4) then
          h=dsqrt(h)*norm
	  rotr= vr(im)/h
          roti=-vi(im)/h
          ih=1
      else
	  rotr=1.d0/norm
          ih=0
      endif
C
C   Rotate and Normalize
      if(ih.eq.1) then
       do  i=1,n
	h    =vr(i)*rotr-vi(i)*roti
	vi(i)=vr(i)*roti+vi(i)*rotr
        vr(i)=h
       end do
      else
       do  i=1,n
	vr(i)=vr(i)*rotr
	vi(i)=vi(i)*rotr
       end do
      end if
      return
      end


      subroutine cbabk2(n,low,ign,scale,m,zr,zi)
      integer i,j,k,m,n,ii,ign,low
      real*8 scale(n),zr(n,m),zi(n,m)
      real*8 s

      if(m.eq.0) go to 200
      if(ign.eq.low) go to 120
      do 110 i=low,ign
      s=scale(i)
c***** left hand eigenvectors are back transformed
c      if the foregoing statement is replaced by
c      s=1.0/scale(i) *****
      do 100 j=1,m
      zr(i,j)=zr(i,j)*s
      zi(i,j)=zi(i,j)*s
100   continue
110   continue
c***** for i=low-1 step -1 until 1, ign+1 step 1 until n do -- *****
120   do 140 ii=1,n
      i=ii
      if(i.ge.low.and.i.le.ign) go to 140
      if(i.lt.low) i=low-ii
      k=scale(i)
      if(k.eq.i) go to 140
      do 130 j=1,m
      s=zr(i,j)
      zr(i,j)=zr(k,j)
      zr(k,j)=s
      s=zi(i,j)
      zi(i,j)=zi(k,j)
      zi(k,j)=s
130   continue
140   continue
200   return
      end

      subroutine cbal(n,ar,ai,low,ign,scale)
      integer i,j,k,l,m,n,jj,ign,low,iexc
      real*8 ar(n,n),ai(n,n),scale(n)
      real*8 c,f,g,r,s,b2,radix
c      real*8 abs
      logical noconv
c***** radix is a machine dependent parameter specifying
c***** the base of the machine floating point representation *****
c*****************************************************************
      radix=2.
      b2=radix*radix
      k=1
      l=n
      go to 100
c***** in-line procedure for row and column exchange *****
20    scale(m)=j
      if(j.eq.m) go to 50
      do 30 i=1,l
      f=ar(i,j)
      ar(i,j)=ar(i,m)
      ar(i,m)=f
      f=ai(i,j)
      ai(i,j)=ai(i,m)
      ai(i,m)=f
30    continue
      do 40 i=k,n
      f=ar(j,i)
      ar(j,i)=ar(m,i)
      ar(m,i)=f
      f=ai(j,i)
      ai(j,i)=ai(m,i)
      ai(m,i)=f
40    continue
50    go to (80,130),iexc
c***** search for rows isolating an eigenvalue
c      and push them down *****
80    if(l.eq.1) go to 280
      l=l-1
c***** for j=l step -1 until 1 do -- *****
100   do 120 jj=1,l
      j=l+1-jj
      do 110 i=1,l
      if(i.eq.j) go to 110
      if(ar(j,i).ne.0.0d0.or.ai(j,i).ne.0.0d0) go to 120
110   continue
      m=l
      iexc=1
      go to 20
120   continue
      go to 140
c***** search for columns isolating an eigenvalue
c      and push them left *****
130   k=k+1
140   do 170 j=k,l
      do 150 i=k,l
      if(i.eq.j) go to 150
      if(ar(i,j).ne.0.0d0.or.ai(i,j).ne.0.0d0) go to 170
150   continue
      m=k
      iexc=2
      go to 20
170   continue
c***** now balance the submatrix in rows k to l *****
      do 180 i=k,l
180   scale(i)=1.0d0
c***** iterative loop for norm reduction *****
190   noconv=.false.
      do 270 i=k,l
      c=0.0d0
      r=0.0d0
      do 200 j=k,l
      if(j.eq.i) go to 200
      c=c+dabs(ar(j,i))+dabs(ai(j,i))
      r=r+dabs(ar(i,j))+dabs(ai(i,j))
200   continue
c***** guard against zero c or r due to underflow *****
      if(c.eq.0.0d0.or.r.eq.0.0d0) go to 270
      g=r/radix
      f=1.0d0
      s=c+r
210   if(c.ge.g) go to 220
      f=f*radix
      c=c*b2
      go to 210
220   g=r*radix
230   if(c.lt.g) go to 240
      f=f/radix
      c=c/b2
      go to 230
c***** now balance *****
240   if((c+r)/f.ge.0.95*s) go to 270
      g=1.0d0/f
      scale(i)=scale(i)*f
      noconv=.true.
      do 250 j=k,n
      ar(i,j)=ar(i,j)*g
      ai(i,j)=ai(i,j)*g
250   continue
      do 260 j=1,l
      ar(j,i)=ar(j,i)*f
      ai(j,i)=ai(j,i)*f
260   continue
270   continue
      if(noconv) go to 190
280   low=k
      ign=l
      return
      end
 
      subroutine CG(n,ar,ai,wr,wi,matz,zr,zi,fv1,fv2,fv3,machep,ierr) 
      integer n,is1,is2,ierr,matz
      real*8 ar(n,n),ai(n,n),wr(n),wi(n),zr(n,n),zi(n,n),
     *       fv1(n),fv2(n),fv3(n),machep

      call cbal(n,ar,ai,is1,is2,fv1)
      call corth(n,is1,is2,ar,ai,fv2,fv3)
      if(matz.ne.0) go to 20
c***** find eigenvalues only *****
c      call comqr(nm,n,is1,is2,ar,ai,wr,wi,ierr)
      go to 50
c***** find both eigenvalues and eigenvectors *****
  20  call comqr2(n,is1,is2,fv2,fv3,ar,ai,wr,wi,zr,zi,machep,ierr)
      if(ierr.ne.0) go to 50
      call cbabk2(n,is1,is2,fv1,n,zr,zi)
   50 continue
      return  
      end
 

      subroutine comqr2(n,low,igh,ortr,orti,hr,hi,wr,wi,zr,zi,
     &                  machep,ierr)
c
      integer i,j,k,l,m,n,en,ii,jj,ll,nn,igh,ip1,
     *        its,low,lp1,enm1,iend,ierr
      real*8 hr(n,n),hi(n,n),wr(n),wi(n),zr(n,n),zi(n,n),
     *       ortr(igh),orti(igh)
      real*8 si,sr,ti,tr,xi,xr,yi,yr,zzi,zzr,norm,machep
      integer min0
      complex*16 z3
c
      ierr=0
c ********** initialize eigenvector matrix **************
      nn=n*n
      call zeroar(nn,zr)
      call zeroar(nn,zi)
      do i=1,n
       zr(i,i)=1.0d0
      end do
c  ********* form the matrix of accumulated transformations
c            from the information left by corth *************
      iend=igh-low-1
      if(iend.le.0) goto 150
c ********** for i=igh-1 step -1 until low+1 do -- **********
      do 140 ii=1,iend
      i=igh-ii
      if(ortr(i).eq.0.0d0.and.orti(i).eq.0.0d0) go to 140
      if(hr(i,i-1).eq.0.0d0.and.hi(i,i-1).eq.0.0d0) go to 140
c ********** norm below is negative of h formed in corth ****
      norm=hr(i,i-1)*ortr(i)+hi(i,i-1)*orti(i)
      ip1=i+1
c
      do 110 k=ip1,igh
      ortr(k)=hr(k,i-1)
      orti(k)=hi(k,i-1)
 110  continue
c
      do 130 j=i,igh
      sr=0.0d0
      si=0.0d0
c
      do 115 k=i,igh
      sr=sr+ortr(k)*zr(k,j)+orti(k)*zi(k,j)
      si=si+ortr(k)*zi(k,j)-orti(k)*zr(k,j)
 115  continue
c
      sr=sr/norm
      si=si/norm
c
      do 120 k=i,igh
      zr(k,j)=zr(k,j)+sr*ortr(k)-si*orti(k)
      zi(k,j)=zi(k,j)+sr*orti(k)+si*ortr(k)
 120  continue
c
 130  continue
c
 140  continue
c ********** create real subdiagonal elements ***************
 150  if(iend.lt.0) goto 180
      l=low+1
c
      do 170 i=l,igh
      ll=min0(i+1,igh)
      if(hi(i,i-1).eq.0.0d0) go to 170
      norm=abs(cmplx(hr(i,i-1),hi(i,i-1)))
      yr=hr(i,i-1)/norm
      yi=hi(i,i-1)/norm
      hr(i,i-1)=norm
      hi(i,i-1)=0.0d0
c
      do 155 j=i,n
      si=yr*hi(i,j)-yi*hr(i,j)
      hr(i,j)=yr*hr(i,j)+yi*hi(i,j)
      hi(i,j)=si
 155  continue
c
      do 160 j=1,ll
      si=yr*hi(j,i)+yi*hr(j,i)
      hr(j,i)=yr*hr(j,i)-yi*hi(j,i)
      hi(j,i)=si
 160  continue
c
      do 165 j=low,igh
      si=yr*zi(j,i)+yi*zr(j,i)
      zr(j,i)=yr*zr(j,i)-yi*zi(j,i)
      zi(j,i)=si
 165  continue
c
 170  continue
c  ********* store roots isolated by cbal *************
 180  do 200 i=1,n
      if(i.ge.low.and.i.le.igh) go to 200
      wr(i)=hr(i,i)
      wi(i)=hi(i,i)
 200  continue 
c
      en=igh
      tr=0.0d0
      ti=0.0d0
c  ********** search for next eigenvalue ***************
 220  if(en.lt.low) go to 680
      its=0
      enm1=en-1
c ******** look for single small sub-diagonal element
c          for l=en step -1 until low -- **************
 240  do 260 ll=low,en
      l=en+low-ll
      if(l.eq.low) go to 300
      if(dabs(hr(l,l-1)).le.
     *       machep*(dabs(hr(l-1,l-1))+dabs(hi(l-1,l-1))
     *      +dabs(hr(l,l))+dabs(hi(l,l)))) go to 300
 260  continue
c  ********* form shift ******************************
 300  if(l.eq.en) go to 660
      if(its.eq.30) go to 1000
      if(its.eq.10.or.its.eq.20) go to 320
      sr=hr(en,en)
      si=hi(en,en)
      xr=hr(enm1,en)*hr(en,enm1)
      xi=hi(enm1,en)*hr(en,enm1)
      if(xr.eq.0.0d0.and.xi.eq.0.0d0) go to 340
      yr=(hr(enm1,enm1)-sr)/2.0d0
      yi=(hi(enm1,enm1)-si)/2.0d0
      z3=sqrt(cmplx(yr**2-yi**2+xr,2.0d0*yr*yi+xi))
      zzr=dreal(z3)
      zzi=dimag(z3)
      if(yr*zzr+yi*zzi.ge.0.0d0) go to 310
      zzr=-zzr
      zzi=-zzi
 310  z3=cmplx(xr,xi)/cmplx(yr+zzr,yi+zzi)
      sr=sr-dreal(z3)
      si=si-dimag(z3)
      go to 340
c ********* form exeptional shift *************************
 320  sr=dabs(hr(en,enm1))+dabs(hr(enm1,en-2))
      si=0.0d0
c
 340  do 360 i=low,en
      hr(i,i)=hr(i,i)-sr
      hi(i,i)=hi(i,i)-si
 360  continue
c
      tr=tr+sr
      ti=ti+si
      its=its+1
c *********** reduce to triangle (rows) ********************
      lp1=l+1
c
      do 500 i=lp1,en
      sr=hr(i,i-1)
      hr(i,i-1)=0.0d0
      norm=dsqrt(hr(i-1,i-1)*hr(i-1,i-1)+hi(i-1,i-1)*hi(i-1,i-1)+sr*sr)
      xr=hr(i-1,i-1)/norm
      wr(i-1)=xr
      xi=hi(i-1,i-1)/norm
      wi(i-1)=xi
      hr(i-1,i-1)=norm
      hi(i-1,i-1)=0.0d0
      hi(i,i-1)=sr/norm
c
      do 490 j=i,n
      yr=hr(i-1,j)
      yi=hi(i-1,j)
      zzr=hr(i,j)
      zzi=hi(i,j)
      hr(i-1,j)=xr*yr+xi*yi+hi(i,i-1)*zzr
      hi(i-1,j)=xr*yi-xi*yr+hi(i,i-1)*zzi
      hr(i,j)=xr*zzr-xi*zzi-hi(i,i-1)*yr
      hi(i,j)=xr*zzi+xi*zzr-hi(i,i-1)*yi
 490  continue
c
 500  continue
c
      si=hi(en,en)
      if(si.eq.0.0d0) go to 540
      norm=abs(cmplx(hr(en,en),si))
      sr=hr(en,en)/norm
      si=si/norm
      hr(en,en)=norm
      hi(en,en)=0.0d0
      if(en.eq.n) go to 540
      ip1=en+1
c
      do 520 j=ip1,n
      yr=hr(en,j)
      yi=hi(en,j)
      hr(en,j)=sr*yr+si*yi
      hi(en,j)=sr*yi-si*yr
 520  continue

c ********* inverse operation (columns) *****************
 540  do 600 j=lp1,en
      xr=wr(j-1)
      xi=wi(j-1)
c
      do 580 i=1,j
      yr=hr(i,j-1)
      yi=0.0d0
      zzr=hr(i,j)
      zzi=hi(i,j)
      if(i.eq.j) go to 560
      yi=hi(i,j-1)
      hi(i,j-1)=xr*yi+xi*yr+hi(j,j-1)*zzi
 560  hr(i,j-1)=xr*yr-xi*yi+hi(j,j-1)*zzr
      hr(i,j)=xr*zzr+xi*zzi-hi(j,j-1)*yr
      hi(i,j)=xr*zzi-xi*zzr-hi(j,j-1)*yi
 580  continue
c
      do 590 i=low,igh
      yr=zr(i,j-1)
      yi=zi(i,j-1)
      zzr=zr(i,j)
      zzi=zi(i,j)
      zr(i,j-1)=xr*yr-xi*yi+hi(j,j-1)*zzr
      zi(i,j-1)=xr*yi+xi*yr+hi(j,j-1)*zzi
      zr(i,j)=xr*zzr+xi*zzi-hi(j,j-1)*yr
      zi(i,j)=xr*zzi-xi*zzr-hi(j,j-1)*yi
 590  continue
c
 600  continue
c
      if(si.eq.0.0d0) go to 240
c
      do 630 i=1,en
      yr=hr(i,en)
      yi=hi(i,en)
      hr(i,en)=sr*yr-si*yi
      hi(i,en)=sr*yi+si*yr
 630  continue
c
      do 640 i=low,igh
      yr=zr(i,en)
      yi=zi(i,en)
      zr(i,en)=sr*yr-si*yi
      zi(i,en)=sr*yi+si*yr
 640  continue
c
      go to 240
c  *******  a root found ************************
 660  hr(en,en)=hr(en,en)+tr
      wr(en)=hr(en,en)
      hi(en,en)=hi(en,en)+ti
      wi(en)=hi(en,en)
      en=enm1
      go to 220
c ********  all roots found. backsubstitute to find
c           vectors of upper triangular form ***********
 680  norm=0.0d0
c
      do 720 i=1,n
c
      do 720 j=1,n
      norm=norm+dabs(hr(i,j))+dabs(hi(i,j))
 720  continue
c
      if(n.eq.1.or.norm.eq.0.0d0) go to 1001
c  ********* for en=n step -1 until 2 do -- ******************
      do 800 nn=2,n
      en=n+2-nn
      xr=wr(en)
      xi=wi(en)
      enm1=en-1
c  ********** for i=en-1 step -1 until 1 do -- ****************
      do 780 ii=1,enm1
      i=en-ii
      zzr=hr(i,en)
      zzi=hi(i,en)
      if(i.eq.enm1) go to 760
      ip1=i+1
c
      do 740 j=ip1,enm1
      zzr=zzr+hr(i,j)*hr(j,en)-hi(i,j)*hi(j,en)
      zzi=zzi+hr(i,j)*hi(j,en)+hi(i,j)*hr(j,en)
 740  continue
c
 760  yr=xr-wr(i)
      yi=xi-wi(i)
      if(yr.eq.0.0d0.and.yi.eq.0.0d0) yr=machep*norm
      z3=cmplx(zzr,zzi)/cmplx(yr,yi)
      hr(i,en)=dble(z3)
      hi(i,en)=dimag(z3)
 780  continue
c
 800  continue
c  ********  end backsubstitution ************************
      enm1=n-1
c  ********  vectors of isolated roots *******************
      do 840 i=1,enm1
      if(i.ge.low.and.i.le.igh) go to 840
      ip1=i+1
c
      do 820 j=ip1,n
      zr(i,j)=hr(i,j)
      zi(i,j)=hi(i,j)
 820  continue
c
 840  continue
c *********  multiply by transformation matrix to give
c            vectors of original full matrix.
c            for j=n step -1 until low+1 do -- ************
      do 880 jj=low,enm1
      j=n+low-jj
      m=min0(j-1,igh)
c
      do 880 i=low,igh
      zzr=zr(i,j)
      zzi=zi(i,j)
c
      do 860 k=low,m
      zzr=zzr+zr(i,k)*hr(k,j)-zi(i,k)*hi(k,j)
      zzi=zzi+zr(i,k)*hi(k,j)+zi(i,k)*hr(k,j)
 860  continue
c
      zr(i,j)=zzr
      zi(i,j)=zzi
 880  continue
c
      go to 1001
c *******  set error -- no convergence to an
c          eigenvalue after 30 iterations **********
1000  ierr=en
1001  continue

      return
      end

      subroutine corth(n,low,igh,ar,ai,ortr,orti)
c
      integer i,j,m,n,ii,jj,la,mpi,igh,kp1,low
      real*8 ar(n,n),ai(n,n),ortr(igh),orti(igh)
      real*8 f,g,h,fi,fr,scale
c      real*8 sqrt,abs
c      complex cmplx
c
      la=igh-1
      kp1=low+1
      if(la.lt.kp1) go to 200
c
      do 180 m=kp1,la
      h=0.0d0
      ortr(m)=0.0d0
      orti(m)=0.0d0
      scale=0.0d0
c  ******** scale column (algol tol then not needed) **********
      do 90 i=m,igh
  90  scale=scale+dabs(ar(i,m-1))+dabs(ai(i,m-1))
c
      if(scale.eq.0.0d0) go to 180
      mpi=m+igh
c  ******** for i=igh step -1 until m do -- *******************
      do 100 ii=m,igh
      i=mpi-ii
      ortr(i)=ar(i,m-1)/scale
      orti(i)=ai(i,m-1)/scale
      h=h+ortr(i)*ortr(i)+orti(i)*orti(i)
 100  continue
c
      g=dsqrt(h)
      f=abs(cmplx(ortr(m),orti(m)))
      if(f.eq.0.0d0) go to 103
      h=h+f*g
      g=g/f
      ortr(m)=(1.0d0+g)*ortr(m) 
      orti(m)=(1.0d0+g)*orti(m)
      go to 105
c
 103  ortr(m)=g
      ar(m,m-1)=scale
c  ********* form (i-(u*ut)/h)*a *********************
 105  do 130 j=m,n
      fr=0.0d0
      fi=0.0d0
c  ********* for i=igh step -1 until m do -- *********
      do 110 ii=m,igh
      i=mpi-ii
      fr=fr+ortr(i)*ar(i,j)+orti(i)*ai(i,j)
      fi=fi+ortr(i)*ai(i,j)-orti(i)*ar(i,j)
 110  continue
c
      fr=fr/h
      fi=fi/h
c
      do 120 i=m,igh
      ar(i,j)=ar(i,j)-fr*ortr(i)+fi*orti(i)
      ai(i,j)=ai(i,j)-fr*orti(i)-fi*ortr(i)
 120  continue
c
 130  continue
c ********* form (i-(u*ut)/h)*a*(i-(u*ut)/h) ***********
      do 160 i=1,igh
      fr=0.0d0
      fi=0.0d0
c  ******** for j=igh step -1 until m do -- ************
      do 140 jj=m,igh
      j=mpi-jj
      fr=fr+ortr(j)*ar(i,j)-orti(j)*ai(i,j)
      fi=fi+ortr(j)*ai(i,j)+orti(j)*ar(i,j)
 140  continue
c
      fr=fr/h
      fi=fi/h
c
      do 150 j=m,igh
      ar(i,j)=ar(i,j)-fr*ortr(j)-fi*orti(j)
      ai(i,j)=ai(i,j)+fr*orti(j)-fi*ortr(j)
 150  continue
c
 160  continue
c
      ortr(m)=scale*ortr(m)
      orti(m)=scale*orti(m)
      ar(m,m-1)=-g*ar(m,m-1)
      ai(m,m-1)=-g*ai(m,m-1)
 180  continue
c
 200  continue

      return
      end

      subroutine dgeco(n,a,ipvt,rcond,z)
      integer n,ipvt(n)
      real*8 a(n,n),z(n),rcond
c
c     dgeco factors a double precision matrix by gaussian elimination
c     and estimates the condition of the matrix.
c
c     if  rcond  is not needed, dgefa is slightly faster.
c     to solve  a*x = b , follow dgeco by dgesl.
c     to compute  inverse(a)*c , follow dgeco by dgesl.
c     to compute  determinant(a) , follow dgeco by dgedi.
c     to compute  inverse(a) , follow dgeco by dgedi.
c
c     on entry
c
c        a       double precision(n, n)
c                the matrix to be factored.
c
c        n     integer
c                the leading dimension of the array  a .
c
c        n       integer
c                the order of the matrix  a .
c
c     on return
c
c        a       an upper triangular matrix and the multipliers
c                which were used to obtain it.
c                the factorization can be written  a = l*u  where
c                l  is a product of permutation and unit lower
c                triangular matrices and  u  is upper triangular.
c
c        ipvt    integer(n)
c                an integer vector of pivot indices.
c
c        rcond   double precision
c                an estimate of the reciprocal condition of  a .
c                for the system  a*x = b , relative perturbations
c                in  a  and  b  of size  epsilon  may cause
c                relative perturbations in  x  of size  epsilon/rcond .
c                if  rcond  is so small that the logical expression
c                           1.0 + rcond .eq. 1.0
c                is true, then  a  may be singular to working
c                precision.  in particular,  rcond  is zero  if
c                exact singularity is detected or the estimate
c                underflows.
c
c        z       double precision(n)
c                a work vector whose contents are usually unimportant.
c                if  a  is close to a singular matrix, then  z  is
c                an approximate null vector in the sense that
c                norm(a*z) = rcond*norm(a)*norm(z) .
c
c     linpack. this version dated 08/14/78 .
c     cleve moler, university of new mexico, argonne national lab.
c
c     subroutines and functions
c
c     linpack dgefa
c     blas daxpy,ddot,dscal,dasum
c     fortran dabs,dmax1,dsign
c
c     internal variables
c
      double precision ddot,ek,t,wk,wkm
      double precision anorm,s,dasum,sm,ynorm
      integer info,j,k,kb,kp1,l
c
c
c     compute 1-norm of a
c
      anorm = 0.0d0
      do 10 j = 1, n
         anorm = dmax1(anorm,dasum(n,a(1,j)))
   10 continue
c
c     factor
c
      call dgefa(n,a,ipvt,info)
c
c     rcond = 1/(norm(a)*(estimate of norm(inverse(a)))) .
c     estimate = norm(z)/norm(y) where  a*z = y  and  trans(a)*y = e .
c     trans(a)  is the transpose of a .  the components of  e  are
c     chosen to cause maximum local growth in the elements of w  where
c     trans(u)*w = e .  the vectors are frequently rescaled to avoid
c     overflow.
c
c     solve trans(u)*w = e
c
      ek = 1.0d0
      do 20 j = 1, n
         z(j) = 0.0d0
   20 continue
      do 100 k = 1, n
         if (z(k) .ne. 0.0d0) ek = dsign(ek,-z(k))
         if (dabs(ek-z(k)) .le. dabs(a(k,k))) go to 30
            s = dabs(a(k,k))/dabs(ek-z(k))
            call dscal(n,s,z)
            ek = s*ek
   30    continue
         wk = ek - z(k)
         wkm = -ek - z(k)
         s = dabs(wk)
         sm = dabs(wkm)
         if (a(k,k) .eq. 0.0d0) go to 40
            wk = wk/a(k,k)
            wkm = wkm/a(k,k)
         go to 50
   40    continue
            wk = 1.0d0
            wkm = 1.0d0
   50    continue
         kp1 = k + 1
         if (kp1 .gt. n) go to 90
            do 60 j = kp1, n
               sm = sm + dabs(z(j)+wkm*a(k,j))
               z(j) = z(j) + wk*a(k,j)
               s = s + dabs(z(j))
   60       continue
            if (s .ge. sm) go to 80
               t = wkm - wk
               wk = wkm
               do 70 j = kp1, n
                  z(j) = z(j) + t*a(k,j)
   70          continue
   80       continue
   90    continue
         z(k) = wk
  100 continue
      s = 1.0d0/dasum(n,z)
      call dscal(n,s,z)
c
c     solve trans(l)*y = w
c
      do 120 kb = 1, n
         k = n + 1 - kb
         if (k .lt. n) z(k) =z(k)+ddot(n-k,a(k+1,k),z(k+1))
         if (dabs(z(k)) .le. 1.0d0) go to 110
            s = 1.0d0/dabs(z(k))
            call dscal(n,s,z)
  110    continue
         l = ipvt(k)
         t = z(l)
         z(l) = z(k)
         z(k) = t
  120 continue
      s = 1.0d0/dasum(n,z)
      call dscal(n,s,z)
c
      ynorm = 1.0d0
c
c     solve l*v = y
c
      do 140 k = 1, n
         l = ipvt(k)
         t = z(l)
         z(l) = z(k)
         z(k) = t
         if (k .lt. n) call daxpy(n-k,t,a(k+1,k),z(k+1))
         if (dabs(z(k)) .le. 1.0d0) go to 130
            s = 1.0d0/dabs(z(k))
            call dscal(n,s,z)
            ynorm = s*ynorm
  130    continue
  140 continue
      s = 1.0d0/dasum(n,z)
      call dscal(n,s,z)
      ynorm = s*ynorm
c
c     solve  u*z = v
c
      do 160 kb = 1, n
         k = n + 1 - kb
         if (dabs(z(k)) .le. dabs(a(k,k))) go to 150
            s = dabs(a(k,k))/dabs(z(k))
            call dscal(n,s,z)
            ynorm = s*ynorm
  150    continue
         if (a(k,k) .ne. 0.0d0) z(k) = z(k)/a(k,k)
         if (a(k,k) .eq. 0.0d0) z(k) = 1.0d0
         t = -z(k)
         call daxpy(k-1,t,a(1,k),z(1))
  160 continue
c     make znorm = 1.0
      s = 1.0d0/dasum(n,z)
      call dscal(n,s,z)
      ynorm = s*ynorm
c
      if (anorm .ne. 0.0d0) rcond = ynorm/anorm
      if (anorm .eq. 0.0d0) rcond = 0.0d0
      return
      end

      subroutine dgedi(n,a,ipvt,det,work,job)
      integer n,ipvt(n),job
      real*8 a(n,n),det(2),work(n)
c
c     dgedi computes the determinant and inverse of a matrix
c     using the factors computed by dgeco or dgefa.
c
c     on entry
c
c        a       double precision(n, n)
c                the output from dgeco or dgefa.
c
c        n     integer
c                the leading dimension of the array  a .
c
c        n       integer
c                the order of the matrix  a .
c
c        ipvt    integer(n)
c                the pivot vector from dgeco or dgefa.
c
c        work    double precision(n)
c                work vector.  contents destroyed.
c
c        job     integer
c                = 11   both determinant and inverse.
c                = 01   inverse only.
c                = 10   determinant only.
c
c     on return
c
c        a       inverse of original matrix if requested.
c                otherwise unchanged.
c
c        det     double precision(2)
c                determinant of original matrix if requested.
c                otherwise not referenced.
c                determinant = det(1) * 10.0**det(2)
c                with  1.0 .le. dabs(det(1)) .lt. 10.0
c                or  det(1) .eq. 0.0 .
c
c     error condition
c
c        a division by zero will occur if the input factor contains
c        a zero on the diagonal and the inverse is requested.
c        it will not occur if the subroutines are called correctly
c        and if dgeco has set rcond .gt. 0.0 or dgefa has set
c        info .eq. 0 .
c
c     linpack. this version dated 08/14/78 .
c     cleve moler, university of new mexico, argonne national lab.
c
c     subroutines and functions
c
c     blas daxpy,dscal,dswap
c     fortran dabs,mod
c
c     internal variables
c
      double precision t
      double precision ten
      integer i,j,k,kb,kp1,l,nm1
c
c
c     compute determinant
c
      if (job/10 .eq. 0) go to 70
         det(1) = 1.0d0
         det(2) = 0.0d0
         ten = 10.0d0
         do 50 i = 1, n
            if (ipvt(i) .ne. i) det(1) = -det(1)
            det(1) = a(i,i)*det(1)
c        ...exit
            if (det(1) .eq. 0.0d0) go to 60
   10       if (dabs(det(1)) .ge. 1.0d0) go to 20
               det(1) = ten*det(1)
               det(2) = det(2) - 1.0d0
            go to 10
   20       continue
   30       if (dabs(det(1)) .lt. ten) go to 40
               det(1) = det(1)/ten
               det(2) = det(2) + 1.0d0
            go to 30
   40       continue
   50    continue
   60    continue
   70 continue
c
c     compute inverse(u)
c
      if (mod(job,10) .eq. 0) go to 150
         do 100 k = 1, n
            a(k,k) = 1.0d0/a(k,k)
            t = -a(k,k)
            call dscal(k-1,t,a(1,k))
            kp1 = k + 1
            if (n .lt. kp1) go to 90
            do 80 j = kp1, n
               t = a(k,j)
               a(k,j) = 0.0d0
               call daxpy(k,t,a(1,k),a(1,j))
   80       continue
   90       continue
  100    continue
c
c        form inverse(u)*inverse(l)
c
         nm1 = n - 1
         if (nm1 .lt. 1) go to 140
         do 130 kb = 1, nm1
            k = n - kb
            kp1 = k + 1
            do 110 i = kp1, n
               work(i) = a(i,k)
               a(i,k) = 0.0d0
  110       continue
            do 120 j = kp1, n
               t = work(j)
               call daxpy(n,t,a(1,j),a(1,k))
  120       continue
            l = ipvt(k)
            if (l .ne. k) call dswap(n,a(1,k),a(1,l))
  130    continue
  140    continue
  150 continue
      return
      end

      subroutine dgefa(n,a,ipvt,info)
      integer n,ipvt(n),info
      real*8 a(n,n)
c
c     dgefa factors a double precision matrix by gaussian elimination.
c
c     dgefa is usually called by dgeco, but it can be called
c     directly with a saving in time if  rcond  is not needed.
c     (time for dgeco) = (1 + 9/n)*(time for dgefa) .
c
c     on entry
c
c        a       double precision(n, n)
c                the matrix to be factored.
c
c        n     integer
c                the leading dimension of the array  a .
c
c        n       integer
c                the order of the matrix  a .
c
c     on return
c
c        a       an upper triangular matrix and the multipliers
c                which were used to obtain it.
c                the factorization can be written  a = l*u  where
c                l  is a product of permutation and unit lower
c                triangular matrices and  u  is upper triangular.
c
c        ipvt    integer(n)
c                an integer vector of pivot indices.
c
c        info    integer
c                = 0  normal value.
c                = k  if  u(k,k) .eq. 0.0 .  this is not an error
c                     condition for this subroutine, but it does
c                     indicate that dgesl or dgedi will divide by zero
c                     if called.  use  rcond  in dgeco for a reliable
c                     indication of singularity.
c
c     linpack. this version dated 08/14/78 .
c     cleve moler, university of new mexico, argonne national lab.
c
c     subroutines and functions
c
c     blas daxpy,dscal,idamax
c
c     internal variables
c
      double precision t
      integer idamax,j,k,kp1,l,nm1
c
c
c     gaussian elimination with partial pivoting
c
      info = 0
      nm1 = n - 1
      if (nm1 .lt. 1) go to 70
      do 60 k = 1, nm1
         kp1 = k + 1
c
c        find l = pivot index
c
         l = idamax(n-k+1,a(k,k)) + k - 1
         ipvt(k) = l
c
c        zero pivot implies this column already triangularized
c
         if (a(l,k) .eq. 0.0d0) go to 40
c
c           interchange if necessary
c
            if (l .eq. k) go to 10
               t = a(l,k)
               a(l,k) = a(k,k)
               a(k,k) = t
   10       continue
c
c           compute multipliers
c
            t = -1.0d0/a(k,k)
            call dscal(n-k,t,a(k+1,k))
c
c           row elimination with column indexing
c
            do 30 j = kp1, n
               t = a(l,j)
               if (l .eq. k) go to 20
                  a(l,j) = a(k,j)
                  a(k,j) = t
   20          continue
               call daxpy(n-k,t,a(k+1,k),a(k+1,j))
   30       continue
         go to 50
   40    continue
            info = k
   50    continue
   60 continue
   70 continue
      ipvt(n) = n
      if (a(n,n) .eq. 0.0d0) info = n
      return
      end



      double precision function dasum(n,dx)
c     takes the sum of the absolute values.
      real*8  dx(n)
      integer i,n
c
      dasum = 0.0d0
      if(n.le.0) return

      do i = 1,n
        dasum = dasum + dabs(dx(i))
      end do

      return
      end

      subroutine daxpy(n,da,dx,dy)
c     constant times a vector plus a vector.
      real*8  dx(n),dy(n),da
      integer i,n
c
      if(n.le.0) return
      if (da.eq.0.0d0) return
      do i = 1,n
        dy(i) = dy(i) + da*dx(i)
      end do
      return
      end

      double precision function ddot(n,dx,dy)
c     forms the dot product of two vectors.
      real*8  dx(n),dy(n)
      integer i,n
c
      ddot = 0.0d0
      if(n.le.0) return

      do i = 1,n
        ddot = ddot + dx(i)*dy(i)
      end do

      return
      end



      subroutine  dscal(n,da,dx)
c     scales a vector by a constant.
c
      real*8 da,dx(n)
      integer i,n

      do i = 1,n
        dx(i) = da*dx(i)
      end do
      return
      end


      subroutine  dswap (n,dx,dy)
c     interchanges two vectors.
      double precision dx(n),dy(n),dtemp
      integer i,n

      if(n.le.0) return
      do i = 1,n
        dtemp = dx(i)
        dx(i) = dy(i)
        dy(i) = dtemp
      end do
      return
      end

      integer function idamax(n,dx)
c     finds the index of element having max. absolute value.
c     jack dongarra, linpack, 3/11/78.
c
      real*8 dx(n),dmax
      integer i,n
c
      idamax = 0
      if(n.lt.1) return
      idamax = 1
      if(n.eq.1) return
       dmax = dabs(dx(1))
      do 30 i = 2,n
         if(dabs(dx(i)).le.dmax) go to 30
         idamax = i
         dmax = dabs(dx(i))
   30 continue

      return
      end

      subroutine zeroar(n,x)
      integer n,i
      real*8 x(n)
      do i=1,n
       x(i)=0.0d0
      end do
      return
      end
