C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
       subroutine optic(b_ew,maxl,NL,b_Cm,MAXGRP,GEOSTR,SFI,Cmatrix,
     &                      chilay,roughlay,VKD,Rmatrix)
C
C ******************************************************************    
C
C                              OPTICS
C
C          Reflectivity calculation of anizotropic multilayers
C          in the Mossbauer case. 1mrad<FI<10mrad, K~1.E-5. 
C
C          (method of G. H. Borzbov et. al. - L. Deak et. al.)
C
C                         Laszlo Deak
C
C                         Budapest, 1995. July
C                         Mainz, 2002. October
C
C                         Khaidukov Yuri
C	                  Budapest, 2003. April-May
C
C ***************************************************************** 
C
C Geometry:
C
C
C               incident        reflected                    
C                          
C                        \    /    
C                         \  /
C                      FI (\/
C      Z=0>           -------------  
C           covering  _____________         [D(1),K(1,I,J)]  
C                                  >Layer#1 [D(2),K(2,I,J)]
C                 1.  .............     
C                           .
C                           .
C                     .............
C Z=NP*Sum(D(L)) NL.  _____________Layer#NL [D(NL-1),K(NL-1,I,J)]
C       I              
C          substrate  _____________         [D(NL),K(NL,I,J)]
C                            `                 ^
C                              `            infinit or finit
C                             refracted
C
C 
C  Variables of system:
C
C  INPUT:
C
C                           covering   layer #L     substrate
C
C            thickneses:         D(1)     D(L)        D(NL)
C
C        susceptibility:
C          (nuclear+electronic)
C                                K(1)     K(L)        K(NL)
C
C         where  L=1..NL  I,J=1,2,3 
C
C  OUTPUT:
C
C         The 2x2 complex reflectivity (or transmitivity) matrix:  R
C
C ******************************************************************
C       
C
       implicit none
C   input:
       integer    NL,NLOG
C     NL: number of layers  (in period)
C   output:
       complex*16 R(2,2)
C   other variables:
       complex*16 C0,C1,CI,M(4,4),ML(4,4),MS(4,4),KL(2,2)

       real*8     sinFI, DL 
       integer    I,J,L,n_rep,GRP,maxl,LMIN,LMAX,Lprev,
     &            GEOSTR(0:2*maxl),MAXGRP,POINTER
	
C
       parameter  (C0=(0.D0,0.D0), C1=(1.D0,0.D0), CI=(0.D0,1.D0))
C**************************************
c
       complex*8  Cmatrix(2,2,maxl),Rmatrix(2,2),chilay(maxl),cz
       real*4     VKD(maxl),SFI,roughlay(maxl),absCmatr,absMatr
       logical    b_Cm(maxl),b_odz,b_first,b_grp
       complex*16 KLprev(2,2),UnMat(2,2) 
       logical    both_dgnl,prev_dgnl,b_ew
 
       parameter  (cz=(0.0,0.0))
C             
       UnMat(1,1)=C1
       UnMat(1,2)=C0	 !Unit matrix
       UnMat(2,1)=C0
       UnMat(2,2)=C1
	 
	 
       sinFI=sin(dble(SFI)) ! the double precision grazing angle 
C
       b_first     =.true.
       POINTER=0                                     
       do GRP=1,MAXGRP      ! loop for the groups
C       To set the characteristic matrix to unit matrix
        do I=1,4
         do J=1,4
          ML(I,J)=C0
         end do
         ML(I,I)=C1
        end do
        b_grp=.false.
C
        LMIN =POINTER+1
        LMAX =POINTER+GEOSTR(2*GRP)
        n_rep=GEOSTR(2*GRP-1)
C
        do L=LMIN,LMAX
         if(VKD(L).gt.0.) then
          b_odz=.false.
          b_grp=.true.
          DL=VKD(L)
          if(b_Cm(L))then
           KL(1,1)=Cmatrix(1,1,L)+chilay(L)
           KL(1,2)=Cmatrix(1,2,L)
           KL(2,1)=Cmatrix(2,1,L)
           KL(2,2)=Cmatrix(2,2,L)+chilay(L)
           absCmatr=real(Cmatrix(1,2,L)*conjg(Cmatrix(1,2,L))+
     &                    Cmatrix(2,1,L)*conjg(Cmatrix(2,1,L)))
           if(absCmatr.lt.1.e-15) b_odz=.true.
          else
           KL(1,1)=chilay(L)
           KL(1,2)=C0
           KL(2,1)=C0
           KL(2,2)=chilay(L)
           b_odz=.true.
          end if          

          call genM(b_odz,sinFI,DL,KL,M)   
c        if(b_ew) write(6,'(a)') 'genM'
c        if(b_ew) write(6,'(4g15.6)')M
C
C Roughness calculation of Y. Khaidukov:
C
	  if(roughlay(L).gt.0.) then
           if(n_rep.gt.1.and.L.eq.Lmin) then  ! In case of repeating layers
	    Lprev=Lmax			      ! we are more interested in the 
	   else                               ! information about roughnesses
                              		      ! in repeated ones. 
	    Lprev=L-1			      ! #Khaidukov
	   endif  ! n_rep.gt.1.and.L.eq.Lmin

	   do I=1,2
	    do J=1,2 
	     if (Lprev.eq.0) then 
	      KLprev(I,J)=C0         ! In case of 1st layer KLprev is for vacuum
	      prev_dgnl=.true.       ! This variable for check the diagonality of prev.
	     endif
                  
             if (Lprev.gt.0) then	   
              prev_dgnl=.not.b_Cm(Lprev)
	      if(b_Cm(Lprev))then
               KLprev(I,J)=Cmatrix(I,J,Lprev)+chilay(Lprev)
              else
               KLprev(I,J)=chilay(Lprev)*UnMat(I,J)
              endif          
	     endif

            end do  ! J
	   end do   ! I
C 
           absMatr=real(dble(
     &              (KLprev(1,2)-KL(1,2))*conjg((KLprev(1,2)-KL(1,2)))+
     &              (KLprev(2,1)-KL(2,1))*conjg((KLprev(2,1)-KL(2,1)))))         


           if(prev_dgnl.and.b_odz) then	      ! If susceptibility of both layers
            both_dgnl=.true.                  ! are diagonal we give true value
           elseif (absMatr.lt.1.e-15)	then  ! for roughness subroutine
	    both_dgnl=.true.
           else
	    both_dgnl=.false.
           endif

           call roughness(both_dgnl,roughlay(L),sinFI,KLprev,KL,M)
c        if(b_ew) write(6,'(a)') ' roughness'
c        if(b_ew) write(6,'(4g15.6)')M
          endif ! Roughness  
C   ***********************	
c        if(b_ew) write(6,'(a)') 'bmat4'
c        if(b_ew) write(6,'(4g15.6)')ML	          
          call mat4(M,ML,ML) 
c        if(b_ew) write(6,'(a)') 'mat4'
c        if(b_ew) write(6,'(4g15.6)')ML	 
         endif ! VKD(L)>0.         
        end do ! L

C     For periodic multilayers the int. propagation matrix is (ML)**n_rep, 
C     where n_rep is the number of periods.
C
C        
        if(n_rep.gt.1.and.b_grp)then
         do I=1,4
          do J=1,4
           M(I,J)=ML(I,J)
          end do
         end do
C        
         NLOG=alog(float(n_rep))/alog(2.0)
C
         do I=1,NLOG
          call mat4(ML,ML,ML)
c        if(b_ew) write(6,'(a)') 'NLOG'
c        if(b_ew) write(6,'(4g15.6)')ML
         end do

         do I=1,n_rep-2**NLOG
          call mat4(M,ML,ML)
c        if(b_ew) write(6,'(a)') '1,n_rep-2**NLOG'
c        if(b_ew) write(6,'(4g15.6)')ML
         end do
C
        end if  ! n_rep>1

        if(b_first) then
         b_first=.false.
         do I=1,4
          do J=1,4
           MS(I,J)=ML(I,J)
          end do
         end do
        else
         call mat4(ML,MS,MS)
        end if         
C
        POINTER=POINTER+GEOSTR(2*GRP)
       end do ! GRP
C
C     Now we have the integreted propagation matrix of the structure: ML
C     Calculation of reflectivity matrix (2x2):
C
        if(b_ew) write(6,'(4g15.6)')MS
	call refl(MS,R,b_ew)
C
        Rmatrix(1,1)=R(1,1)
        Rmatrix(1,2)=R(1,2)
        Rmatrix(2,1)=R(2,1)
        Rmatrix(2,2)=R(2,2)
C
	return
	end
C
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
	subroutine mat2(A,B,C)
C
C ********************************************************************
C
C                    Matrix multiplication (for 2X2)
C
C *******************************************************************
C
      implicit none
C
	complex*16 A(2,2),B(2,2),C(2,2),F(2,2)
        integer    I,J,L
C
C       A*B=C

	do I=1,2
	 do L=1,2
	  F(I,L)=(0.D0,0.D0)
	  do J=1,2
	   F(I,L)=F(I,L)+A(I,J)*B(J,L)
	  end do
	 end do
	end do
C
	do I=1,2
	 do J=1,2
	  C(I,J)=F(I,J)
	 end do
	end do
C
	return
	end
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	subroutine inv2(A,B)
C
C ******************************************************************
C
C                    Inverse  matrix (for 2X2)
C
C ******************************************************************
C
      implicit none
C
	complex*16 A(2,2),B(2,2),rdetA
        complex*16 C1
        parameter(C1=(1.D0,0.D0))    

C       B=A**(-1)
C
        
	rdetA=A(1,1)*A(2,2)-A(1,2)*A(2,1)
         B(1,1)= A(2,2)/rdetA
         B(2,2)= A(1,1)/rdetA
         B(1,2)=-A(1,2)/rdetA
         B(2,1)=-A(2,1)/rdetA

	return
	end
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	subroutine sqrt2(A,B)
C
C ******************************************************************
C
C                     sqrt matrix (for 2X2)
C
C ******************************************************************
C
      implicit none
C
	complex*16 A(2,2),B(2,2),DetA,TrA,AA,sqDetA
	real*8     imDetA
C
C       B     = sqrt(A) =[A+I*sqrt(DetA)]/sqrt[TrA+2*sqrt(DetA)]
C       
        DetA  = A(1,1)*A(2,2)-A(1,2)*A(2,1)
	sqDetA= sqrt(DetA)
        TrA   = A(1,1)+A(2,2)
C
	imDetA= dimag(DetA)
	if(imDetA.gt.0.D0) sqDetA=-sqDetA
C
	AA    = 1.D0/sqrt(TrA+2.D0*sqDetA)
C
	B(1,1)= (A(1,1)+sqDetA)*AA
	B(2,2)= (A(2,2)+sqDetA)*AA
	B(1,2)= A(1,2)*AA
	B(2,1)= A(2,1)*AA
C
	return
	end
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
	subroutine mat4(A,B,C)
C
C *****************************************************************
C
C                    Matrix multiplication (for 4X4)
C
C ******************************************************************
C
      implicit none
C
	complex*16 A(4,4),B(4,4),C(4,4),F(4,4)
        integer    I,J,L
C
C       A*B=C

	do I=1,4
	  do L=1,4
	    F(I,L)=(0.D0,0.D0)
	    do J=1,4
	      F(I,L)=F(I,L)+A(I,J)*B(J,L)
	    end do
	  end do
	end do
C
	do I=1,4
	  do J=1,4
	    C(I,J)=F(I,J)
	  end do
	end do
C 	
	return
	end
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
	subroutine exp2(N)
C
C ****************************************************************
C
C                    Matrix exponential (for 2X2)
C
C ****************************************************************
C
      implicit none
C
      complex*16  N(2,2),hTrN,detN,C1,C0
      complex*16  sindN,cosdN,exphTrN
      real*8      aLIMIT,rehTrN,absCmatr
      parameter  (aLIMIT=2.D1)
      parameter  (C0    =(0.D0,0.D0))
      parameter  (C1    =(1.D0,0.D0))
C
      hTrN     = 0.5D0*(N(1,1)+N(2,2))
      rehTrN   = dreal(hTrN)
      absCmatr=dreal(N(1,2)*conjg(N(1,2))+
     &               N(2,1)*conjg(N(2,1)))


C
      if(absCmatr.lt.1.e-15)	then          !
	N(1,1)=exp(N(1,1))		      !Case of diagonal matrix
	N(1,2)=0.			      !#Khaidukov
	N(2,1)=0.			      !
	N(2,2)=exp(N(2,2))		      !

	
	else if(abs(rehTrN).lt.aLIMIT)then
       N(1,1)=N(1,1)-hTrN
       N(2,2)=N(2,2)-hTrN
C
       detN    = sqrt(N(1,1)*N(2,2)-N(1,2)*N(2,1))
       sindN   = sin(detN)
       cosdN   = cos(detN)
       exphTrN = exp(hTrN)
C
       N(1,1)  = exphTrN*(cosdN+sindN*N(1,1)/detN)
       N(2,2)  = exphTrN*(cosdN+sindN*N(2,2)/detN)
       N(1,2)  = exphTrN*sindN*N(1,2)/detN
       N(2,1)  = exphTrN*sindN*N(2,1)/detN
      end if ! abs(real(exp(hTrN))) < aLIMIT
	
     		 
C
      return
      end
C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
	subroutine refl(M,R,b_ew)
C
C *******************************************************************
C
C                 Calculation of reflectivity matrix (2X2)
C
C ******************************************************************
C
      implicit none
C
	complex*16 M(4,4),R(2,2),AA,BB,CC,DD
	complex*16 R1(2,2),R2(2,2),R11(2,2)
	integer    I,J
        logical b_ew
C    Definition of 2x2 block matrices:
C
	do I=1,2
	  do J=1,2
	    AA= M(I,J)
  	    BB= M(I,J+2)
	    CC= M(I+2,J)
	    DD= M(I+2,J+2)
            R1(I,J)= AA-BB-CC+DD
            R2(I,J)= AA+BB-CC-DD
	  end do
	end do

 	 call inv2(R1,R11)
         call mat2(R11,R2,R)

	return
	end
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
	subroutine genM(b_odz,sinFI,DL,KL,M)
C
C ******************************************************************
C
C            Definition of the integral propagation matrix  
C                   (see  Eq. (3.20) in the paper)
C
C *****************************************************************
C
      implicit none
C
      integer    I,J
      complex*16 M(4,4),exppF(2,2),expmF(2,2),
     &           FL(2,2),KL(2,2) ,!AA(2,2),BB(2,2),CC(2,2),
     &           invF(2,2),shF(2,2),M12(2,2),M21(2,2),FF(2,2),
     &           FFF,C1,C0,CI,sinFFF,FF1
      real*8     aimFFF,aLIMIT,DL,sinFI,sin2FI
C      real*4     ROUGH1,ROUGH2
	logical    b_odz
C
      parameter(C0=(0.D0,0.D0))
      parameter(CI=(0.D0,1.D0))
      parameter(C1=(1.D0,0.D0))
      parameter(aLIMIT=2.D1)
C
C      Definition of F and X ( Eq. (3.20) L.Deak et al, Phys. Rev B 1996)
C
C     ROUG1  = ROUGH1
C      ROUG2  = ROUGH2
      sin2FI = sinFI*sinFI
C
      FF(1,1)=-sin2FI-KL(1,1)
      FF(1,2)=-KL(1,2)
      FF(2,1)=-KL(2,1)
      FF(2,2)=-sin2FI-KL(2,2)
C
C        b_odz==(FF(2,1).eq.C0).and.(FF(1,2).eq.C0)
      if(b_odz)then
       FF(1,1)=sqrt(FF(1,1))
       FF(2,2)=sqrt(FF(2,2))
      else
       call sqrt2(FF,FF)
      endif
C
      do I=1,2
       do J=1,2
        FL(I,J)=DL*FF(I,J)
       end do
      end do
C
      do I=1,2
       do J=1,2
        exppF(I,J) =FL(I,J)
        M(I,J)     =C0
        M(I+2,J+2) =C0
        M(I,J+2)   =C0
        M(I+2,J)   =C0
       end do
      end do
C        b_odz==(FL(2,1).eq.C0).and.(FL(1,2).eq.C0)
      if(b_odz)then
       do J=1,2
        FF1   =FF(J,J)
        FFF   =FL(J,J)*CI        
        aimFFF=dimag(FFF)
        if(aimFFF.gt.aLIMIT)then
         M(J,J)    = C1
         M(J+2,J+2)= C1
         M(J,J+2)  =-CI*FF1/sinFI
         M(J+2,J)  = CI/FF1*sinFI      ! aimFFF -> +infinit
C         write(*,*)'+inf'
        else
         if(aimFFF.lt.-aLIMIT)then
          M(J,J)    = C1
          M(J+2,J+2)= C1
          M(J,J+2)  = CI*FF1/sinFI
          M(J+2,J)  =-CI/FF1*sinFI     ! aimFFF -> -infinit
C          write(*,*)'-inf'
         else
          sinFFF    = sin(FFF)
          M(J,J)    = cos(FFF)
          M(J+2,J+2)= M(J,J)
          M(J,J+2)  =-sinFFF*FF1/sinFI
          M(J+2,J)  = sinFFF/FF1*sinFI ! general case
C          write(*,*)'general'
         end if
        end if
       end do 
C      
      else

       call inv2(FF,invF)
       call exp2(exppF)
       call inv2(exppF,expmF)
C
       do I=1,2
        do J=1,2
         M(I,J)    = 0.5D0*(exppF(I,J)+expmF(I,J))
         M(I+2,J+2)= M(I,J)
         shF(I,J)  = 0.5D0*(exppF(I,J)-expmF(I,J))
        end do
       end do
C
       call mat2(FF,shF,M12)
       call mat2(invF,shF,M21)
C
       do I=1,2
        do J=1,2
         M(I,J+2) = -M12(I,J)/sinFI*CI
         M(I+2,J) =  M21(I,J)*sinFI*CI
        end do
       end do
C
      endif ! b_odz == KL diagonal
C      
cC        roughness
c      if((ROUGH1.gt.0.).or.(ROUGH2.gt.0.))then
cC        
c       R2min=(ROUG1**2-ROUG2**2)/2.D0
c       R2plu=(ROUG1**2+ROUG2**2)/2.D0
cc       write(*,*)R2min,R2plu
c       do I=1,2
c	do J=1,2
c	 AA(I,J)= M(I,J)
c  	 BB(I,J)= M(I,J+2)
c	 CC(I,J)= M(I+2,J)
cC	 DD(I,J)= M(I+2,J+2)
c	end do
c       end do
cC
c       call mat2(AA,KL,AA)
c       call mat2(BB,KL,BB)
c       call mat2(CC,KL,CC)
cC
c       do I=1,2
c	do J=1,2
c	 M(I,J)    = M(I,J)+R2min*AA(I,J)
c  	 M(I,J+2)  = M(I,J+2)-R2plu*BB(I,J)
c	 M(I+2,J)  = M(I+2,J)+R2plu*CC(I,J)
c	 M(I+2,J+2)= M(I+2,J+2)-R2min*AA(I,J)
c	end do
c       end do
C
c      endif   ! roughess  
cC
      return
      end

C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	subroutine roughness(both_dgnl,sigma,A0,A1,A2,M)
C ******************************************************************

C  This subroutine updates integrated matrix of distibution with roughness on
C	 the top border of a given layer. 
C      M->Mupd, where Mupd=exp(sigma**2)*[W1,W2]*0.5*M. For the details see 
C      R. Rohlsberger, Hyperfine Inreractions 123/124 (1999) 301-325
C      ------------------------------------------------------------
C	 Differential matrix of distribution is  represented like
C		      0		  I*Ao+Ai/Ao
C      Wi   =	  
C                    I*Ao		0
     	      
C      ---------------------------------------------------------------     
       implicit none

C      Input variables	
       logical    both_dgnl
       real*4     sigma
       real*8     A0
       complex*16 A1(2,2),A2(2,2),M(4,4)

C      Output
       complex*16 Mupd(4,4)

C      Other varaibles
       real*8     sigma2
       complex*16 exp_Dp(2,2),exp_Dm(2,2),exp_scalar(4),C1,C0,
     &            dA_scal(2),C_scal(2),C(2,2),dA(2,2)
       integer    I,J,K

C      Matrixes
       complex*16 TMP(2,2),C2(2,2)

       parameter  (C1=(1.D0,0.D0))
       parameter  (C0=(0.0,0.0))	
  
       sigma2=sigma
       sigma2=sigma2*sigma2

       do I=1,4
        do J=1,4
         Mupd(I,J)=C0
        end do
       end do
     
       if(both_dgnl) then
	do I=1,2
         dA_scal(I)      = A2(I,I)-A1(I,I)
         C_scal(I)       = 2.D0*(2.D0*A0*A0+A2(I,I)+A1(I,I)) ! C=2*(2*A0*A0+A1+A2)
         exp_scalar(I)   = dA_scal(I)*(C1/C_scal(I))*
     &                     (-exp(-0.5D0*sigma2*C_scal(I))+C1)
         exp_scalar(I)   = exp(exp_scalar(I))
         exp_scalar(I+2) = C1/exp_scalar(I)
	end do ! I

C        do I=1,4
C         do J=1,4
C          Mupd(I,J) =  exp_scalar(J)*M(I,J)
C         end do
C	end do

        Mupd(1,1) =  exp_scalar(1)*M(1,1)     ! instead of the above loop it is better to use
        Mupd(2,2) =  exp_scalar(2)*M(2,2)     ! the direct definitions, because we have only
        Mupd(3,1) =  exp_scalar(1)*M(3,1)     ! 8 complex products insted of 16
        Mupd(4,2) =  exp_scalar(2)*M(4,2)     ! # L. Deak

        Mupd(1,3) =  exp_scalar(3)*M(1,3)
        Mupd(2,4) =  exp_scalar(4)*M(2,4)
        Mupd(3,3) =  exp_scalar(3)*M(3,3)
        Mupd(4,4) =  exp_scalar(4)*M(4,4)

       elseif(.not.both_dgnl)  then             
        C(1,1)=2.D0*(2.D0*A0*A0+A1(1,1)+A2(1,1))
        C(2,2)=2.D0*(2.D0*A0*A0+A1(2,2)+A2(2,2))             ! C=2*(2*A0*A0+A1+A2)
        C(1,2)=2.D0*(A1(1,2)+A2(1,2))
        C(2,1)=2.D0*(A1(2,1)+A2(2,1))

	do I=1,2
	 do J=1,2
          dA(I,J)=A2(I,J)-A1(I,J)                ! A=A2-A1
          TMP(I,J)=-0.5D0*sigma2*C(I,J)          ! TMP=-0.5*sigma**2*C
         end do
        end do

        call exp2(TMP)                           ! TMP=exp(-0.5*sigma**2*C)
        call inv2(C,C2)                          ! C2=1/C
        call mat2(dA,C2,C2)                      ! C2=dA*1/C

        TMP(1,1)= C1-TMP(1,1)
        TMP(1,2)=-TMP(1,2)
        TMP(2,1)=-TMP(2,1)                       ! TMP=I-exp(-0.5*sigma**2*C)
        TMP(2,2)= C1-TMP(2,2)

        call mat2(C2,TMP,exp_Dp)                 ! expDp=dA*1/C*(I-exp(-0.5*sigma**2*C))
        call exp2(exp_Dp)                        ! expDp=exp(dA*1/C*(I-exp(-0.5*sigma**2*C)))
        call inv2(exp_Dp,exp_Dm)

        do I=1,2             
         do J=1,2
	  do K=1,2
           Mupd(I,J)     = Mupd(I,J)    +  M(I,K)*exp_Dp(K,J)
           Mupd(I,J+2)   = Mupd(I,J+2)  +  M(I,K+2)*exp_Dm(K,J)
           Mupd(I+2,J)   = Mupd(I+2,J)  +  M(I+2,K)*exp_Dp(K,J)
           Mupd(I+2,J+2) = Mupd(I+2,J+2)+  M(I+2,K+2)*exp_Dm(K,J)	  
          end do
         end do
        end do

       end if                                    ! both_dgnl

       do I=1,4
        do J=1,4
         M(I,J)=Mupd(I,J)
        end do
       end do
      
       return
       end




