 	subroutine n_diffuse(maxl,NL,b_Cm,MAXGRP,GEOSTR,Cmatrix,VKD,
     &                     rough,SFi_inc,SFi_fin,b_plus,Rmatrix,
     &                 domain,k_wave,DWBA,ksi,Imatrix)

!************************************************************************************************************
!               The subroutine calculates intensity of diffuse scattering of a 
!
!            antiferromagnetically coupled correlated magnetic  domains (exp. 2.18)       
!
!      Incoming angle - SFi_inc, Outgoing angle - SFi_fin, correlation length - ksi.
!
!      Information about in which layers domain exists is situated in variable domain.
!
!     Domain(i)=.true. if the i-th layer has domains.
!
!                           Budapest, september 2004
!
!                               Khaidukov Yu.
!************   Variables, etc********************************************************************************
      implicit none
C INPUT:      
C
	integer maxl 
        integer NL,GEOSTR(0:2*maxl),MAXGRP
	logical b_Cm(maxl),b_plus,DWBA,domain(maxl)
	real*4 SFi_inc,SFi_fin,k_wave,VKD(maxl),
     &         rough(maxl),ksi
	complex*8 Cmatrix(2,2,maxl) 


C OUTPUT:
	complex*8 Imatrix(2,2) 

C Other 
c	real*8 mag_const,nuc_const
	complex*8 Im1,Rmatrix(2,2)
c	integer N1,N2,NR
c	real*8 Bx,By,Bz	
        real*8 K1,pi
	parameter (Im1=(0.D0,1.D0),pi=3.14159265358D0)
	real*4 SQ
c	integer fi_points,answer

	integer I_DMAX
	parameter  (I_DMAX=1024)   
        complex*8 Tmatrix(2,2,I_DMAX)
c	complex*16 Amatrix(2,2,I_DMAX),Bmatrix(2,2,I_DMAX)

C**************************************

 	SQ=2.0*k_wave*sin(SFi_inc)										    
  !    call  n_optics(maxl,NL,b_Cm,MAXGRP,GEOSTR,SQ,Hextz,Cmatrix,   
   !  &               VKD,rough,b_plus,.not.b_plus,Rmatrix)	       

      K1=dble(k_wave*sin(SFi_fin))
      call  n_field(maxl,I_DMAX,NL,b_Cm,MAXGRP,GEOSTR,SQ,K1,Cmatrix, 
     &               VKD,rough,Rmatrix,domain,Tmatrix)    
                                     ! Fourier for each layer  
	 
!	call Fourier_for_layers(I_DMAX,Tmatrix,k_wave,SFi_fin,
!    &	    MAXGRP,GEOSTR,VKD,domain,Four_matrix)


      call  n_diffuse_base(maxl,NL,MAXGRP,GEOSTR,SFi_inc,SFi_fin,b_plus,
     &                     k_wave,domain,ksi,DWBA,Tmatrix,Imatrix)

      RETURN
      END


C *******************************************************************    
      subroutine  n_diffuse_base(maxl,NL,MAXGRP,GEOSTR,SFi_inc,SFi_fin,
     &                      b_plus,k_wave,domain,
     &                  	  ksi,DWBA,Tmatrix,I_off)
 
C*******************************************************************    
C                               OPTICS
C
C          Calculation of intensity of diffuse scattering for 
C		 magnetic domain roughness. Black-n-white 
C          correlated domains, neutron case. 
C
C          (method of G. H. Borzbov et. al. - L. Deak et. al.)
C
C
C                         Budapest,  May, November 
C                                     September 2004     
C
C	                    Khaidukov Yuri
C ***************************************************************** 
C
C Geometry:
C
C
C               incident        reflected                    
C                          
C                        \    /    
C                         \  /
C                  FI_inc (\/) FI_final
C      Z=0>           -------------  
C           covering  _____________         [D(1),K(1,I,J)]  
C                                  >Layer#1 [D(2),K(2,I,J)]
C                 1.  .............     
C                           .
C                           .
C                     .............
C Z=NP*Sum(D(L)) NL.  _____________Layer#NL [D(NL-1),K(NL-1,I,J)]
C       I              
C          substrate  _____________         [D(NL),K(NL,I,J)]
C                            `                 ^
C                              `            infinit or finit
C                             refracted
C
C 
C  Variables of system:
C
C  INPUT:
C
C                           covering   layer #L     substrate
C
C            thickneses:         D(1)     D(L)        D(NL)
C
C        susceptibility:
C          (nuclear+magnetic)
C                                K(1)     K(L)        K(NL)
C
C       	 effect_rough       sigma(1)   sigma(L)    sigma(NL)
C  (struct+magnetic rough)
C		 magn_rough      mag_sigma(1)  mag_sigma(2)	mag_sigma(NL)
C (rms of domain angle distrib)
C	   mag_sigma(i)->0
C lateral length of correl-n	 ksi(1)     ksi(2)       ksi(NL)
C   where  L=1..NL  I,J=1,2,3 
C --------
C      SFI_inc,SFI_final - incident angle and angle of scattered wave
C      r                    sample-detector distance
C  	 A                    area of sample
C  
C
C
C  OUTPUT:
C
C         Intensity of diffusely scattered wave from the above sample 
C		which goes under angle SFI-final
C
C ******************************************************************
      implicit none
C INPUT:      
C
      integer maxl,NL,GEOSTR(0:2*maxl),MAXGRP
	logical b_plus
	real*4 SFi_inc,SFi_fin,k_wave,
     &         ksi
!	complex*8 Cmatrix(2,2,maxl) 
	logical DWBA,domain(maxl)

	integer I_DMAX
	real*4 	VKDMAX
	parameter  (VKDMAX=7.30416e3,I_DMAX=1024)
      complex*8 Tmatrix(2,2,I_DMAX)

C OUTPUT:
	complex*8 I_off(2,2)

C Other variables
	logical b_first,b_grp,b_grp1,mag_in_L_plus,mag_in_L1_plus,
     *b_first1,both_plus
	integer POINTER,GRP,GRP1,LMIN,LMAX,n_rep,L,n,POINTER1,
     *LMIN1,LMAX1,n_rep1,n1,L1
	real*8 Kz,Qx,pi,C
 
	integer I1,I2,ii,jj

	complex*8 T_kz(2,2),T_kz2(2,2),dI(2,2)
C**************************************
	NL=5 !Unused variable byt I must put it for EFFI

	pi=atan(1.)*4. 
	      Kz=dble(k_wave*sin(dble(SFi_fin)))  !perpenidular component of dif. wave
	Qx=dble(k_wave*(cos(dble(SFi_fin))-cos(dble(SFi_inc))))!parallel component of Q

C***************** Sum over layers************
	 



	I_off(1,1)=(0.0D0,0.0D0)  
	I_off(1,2)=(0.0D0,0.0D0)  
	I_off(2,1)=(0.0D0,0.0D0)  
	I_off(2,2)=(0.0D0,0.0D0)  

      mag_in_L_plus=.true.   !magnetization in layer L is paralel
	I1=0

C First sum over L
       b_first     =.true.
	 POINTER=0
                                    
       do GRP=1,MAXGRP      ! loop for the groups
        b_grp=.false.
C
        LMIN =POINTER+1
        LMAX =POINTER+GEOSTR(2*GRP)
        n_rep=GEOSTR(2*GRP-1)
C
	n=1
   
      DO WHILE (n.LE.n_rep)
	n=n+1
        do L=LMIN,LMAX
	  if(domain(L)) then    !  

	       I1=I1+1
	            DO ii=1,2
	             DO jj=1,2
	               T_kz(ii,jj)=Tmatrix(ii,jj,I1)
	             END DO
	            END DO
	               
	mag_in_L1_plus=.true.   !magnetization in layer L1 is paralel

C Second sum over L1

	      b_first1=.true.
            POINTER1=0
	      I2=0
C            
		  do GRP1=1,MAXGRP      ! loop for the groups
             b_grp1=.false.

	        LMIN1 =POINTER1+1
              LMAX1 =POINTER1+GEOSTR(2*GRP1)
              n_rep1=GEOSTR(2*GRP1-1)
C
	        n1=1
             DO WHILE (n1.LE.n_rep1)
	       n1=n1+1

           	        do L1=LMIN1,LMAX1

	                if(domain(L1))  then  !
	                
!	                call fourier4field (Tmatrix,DT,d1_1,d2_1,Kz,
 !    *				DWBA,T_kz2)			
 	       I2=I2+1
	            DO ii=1,2
	             DO jj=1,2
	               T_kz2(ii,jj)=Tmatrix(ii,jj,I2)
	             END DO
	            END DO

	            if (mag_in_L_plus.and.mag_in_L1_plus) then
	               both_plus=.true.
	             else if(.not.mag_in_L_plus.and..not.mag_in_L1_plus)
     *			    then
	               both_plus=.true.
	               else
	               both_plus=.false.
	               endif

!					print *, L,L1 
	                call 
     *				corellate(Qx,ksi,1.0,both_plus,C)
              	    call contribution (T_kz,C,T_kz2,dI)   
!					print *, l,mag_in_L_plus,l1,mag_in_L1_plus,
!	*				both_plus
					 I_off(1,1)=I_off(1,1)+dI(1,1)
					 I_off(1,2)=I_off(1,2)+dI(1,2)    
	                 I_off(2,1)=I_off(2,1)+dI(2,1)  
	                 I_off(2,2)=I_off(2,2)+dI(2,2)  

				   mag_in_L1_plus=.not.mag_in_L1_plus

					endif ! VKD(L1) > 0. and ksi(L1)>0
	                end do ! L1
		  
		  END DO  ! n1<=n1_rep
	      POINTER1=POINTER1+GEOSTR(2*GRP1)
	      end do ! GRP1

      	  mag_in_L_plus=.not.mag_in_L_plus


         endif ! VKD(L) > 0. and ksi>0        
!	   print *, d1,d2
         end do ! L
	END DO  ! n<=n_rep

        POINTER=POINTER+GEOSTR(2*GRP)
       end do ! GRP

!      if (IMAG(I).GT.0)  print *,REAL(I),IMAG(I)

	return
	end
C  *****************************************************************
C    Contribution to diffuse scattering of a layer for 
C    a given polarization.      
C   		dI=Tr(T1_transposed*C*T2*ro)/r**2, C~I
C  *****************************************************************
	subroutine contribution (T1,C,T2,dI)
	implicit none
	complex*8 T1(2,2),T2(2,2),dI(2,2)
      real*8 C

	dI(1,1)=C*(conjg(T1(1,1))*T2(1,1)+conjg(T1(2,1))*T2(2,1))
      dI(1,2)=C*(conjg(T1(1,1))*T2(1,2)+conjg(T1(2,1))*T2(2,2))
	dI(2,1)=C*(conjg(T1(1,2))*T2(1,1)+conjg(T1(2,2))*T2(2,1))
	dI(2,2)=C*(conjg(T1(1,2))*T2(1,2)+conjg(T1(2,2))*T2(2,2))
     


      return
      end

C  *****************************************************************
C    Correlation function of susceptibilities in  layers
C         
C   	C~+-I*sigma_magn*(ksi*ksi*pi2)/((1+(Qx*ksi)**2)**1.5)
C  *****************************************************************
	subroutine corellate(Qx,ksi,sigma,both_plus,C)
	implicit none
	logical both_plus
	real*4 ksi,sigma
	real*8 Qx,ksiD,pi2,pr,sigmaD,C

	ksiD=ksi
	sigmaD=sigma
      pi2=atan(1.)*8.
	pr=(Qx*ksiD)*(Qx*ksiD)
	IF (both_plus) then 
	C=sigmaD*(ksiD*ksiD*pi2)/((1.D0+pr)**1.5)
!	C=1.0
	ELSE
	C=-sigmaD*(ksiD*ksiD*pi2)/((1.D0+pr)**1.5)  
!	C=-1.0
!      C=1.0 
	END IF
! 	C=1.
! 	print *, C
      return
      end



C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      subroutine n_field(maxl,I_DMAX,NL,b_Cm,MAXGRP,GEOSTR,SQ,K1,
     &        Cmatrix,VKD,roughlay,Rmatrix,domain,Tmatrix)
C*********************************************************************************************************************
C      This subroutine calculates Fourier transform (2.12) for each layer where domains exists.  
C
C     Field of incident wave is calculated using Rmatrix for SQ=2k(sin(theta_in)). K1=k(sin(theta_out)
C
C                                        Budapest 2004, 
C                             
C                                        Khaidukov Yu
C
C
C
C
CC*********************************************************************************************************************
 
 
       implicit none
C   input:
       integer    NL,maxl,I_DMAX
	 logical domain(maxl)
C     NL: number of layers  (in period)
       complex*16 T(2,2)
C   other variables:
       complex*16 C0,C1,CI,M(4,4),ML(4,4),KL(2,2),R(2,2)
       real*8     QD, DL,K1
       integer    I,J,L,n_rep,GRP,LMIN,LMAX,
     &            GEOSTR(0:2*maxl),MAXGRP,POINTER,I_D
c       real*8     Dper(maxl),Dgrp(maxl) 

       parameter  (C0=(0.D0,0.D0), C1=(1.D0,0.D0), CI=(0.D0,1.D0))
C**************************************

       complex*8  Cmatrix(2,2,maxl),Rmatrix(2,2),cz,
     &            Tmatrix(2,2,I_DMAX)
      complex*16  A(2,2),B(2,2)
       real*4     VKD(maxl),SQ,roughlay(maxl),absCmatr,
     &            VKDMAX
       logical    b_Cm(maxl),b_odz,b_first,b_grp
!       parameter  (cz=(0.0,0.0),VKDMAX=7.30416e3)
       parameter  (cz=(0.0,0.0),VKDMAX=7.30416e4)

C********* Variables added by Khaidukov for neutron case
	integer Lprev,Lnext,n
	logical prev_dgnl,both_dgnl,both_NONdgnl
	complex*16 KLprev(2,2)
	real*8 DabsMatr,z1,z2
             
C---------------------------------------------
	NL=5 !Unused for this program variable

	 QD=dble(SQ)*0.5D0 

       R(1,1) = Rmatrix(1,1)
       R(1,2) = Rmatrix(1,2)
       R(2,1) = Rmatrix(2,1)
       R(2,2) = Rmatrix(2,2)
C 
       b_first     =.true.

C       To set the characteristic matrix to unit matrix
        do I=1,4
         do J=1,4
          ML(I,J)=C0
         end do
         ML(I,I)=C1
        end do

	I_D=0

       POINTER=0                                     
      z1=0.0D0  !First 
      z2=0.0D0  !and the second coordinate for each layer

       do GRP=1,MAXGRP      ! loop for the groups
        b_grp=.false.
C
        LMIN =POINTER+1
        LMAX =POINTER+GEOSTR(2*GRP)
        n_rep=GEOSTR(2*GRP-1)
C
      	n=1
   
      DO WHILE (n.LE.n_rep)
	n=n+1
        do L=LMIN,LMAX 
	  if(VKD(L).gt.0.and.VKD(L).lt.1.e30) then

C          b_odz=.false.                              
	    b_odz=.not.b_Cm(L)
		b_grp=.true.  			 
            DL=VKD(L)

	      z2=z1+DL
            KL(1,1)=Cmatrix(1,1,L)
            KL(1,2)=Cmatrix(1,2,L)
            KL(2,1)=Cmatrix(2,1,L)
            KL(2,2)=Cmatrix(2,2,L)
            absCmatr=real(Cmatrix(1,2,L)*conjg(Cmatrix(1,2,L))+
     &                    Cmatrix(2,1,L)*conjg(Cmatrix(2,1,L)))
           if(absCmatr.lt.1.e-15) then
		 b_odz=.true.
	     end if

          IF (domain(L)) THEN !If there are domains in this layer store inform
   
      	call trans(QD,KL,R,ML,A,B)  ! A ne M li dolzno bit vmest ML??? Kh
          call    analytic_fourier(z1,z2,QD,K1,KL,A,B,T)
          I_D=I_D+1
           Tmatrix(1,1,I_D)=T(1,1)
	     Tmatrix(1,2,I_D)=T(1,2)
	     Tmatrix(2,1,I_D)=T(2,1)
	     Tmatrix(2,2,I_D)=T(2,2)

           
                   END IF     ! Domains

          call genM(b_odz,QD,DL,KL,M)  !

C************* Roughness************
	IF (roughlay(L).GT.0) THEN
           if (n_rep.GT.1.and.L.eq.Lmin) then  ! In case of repeating layers
	     Lprev=Lmax	                       ! we are more interested in the 
	     else                              !information about roughnesses
                              		       !in repeated ones. 
	     Lprev=L-1			               !#Khaidukov
	     END IF

          if(n_rep.GT.1.and.L.eq.LMAX) then  !
           Lnext=LMIN         ! For peiodical structure the rougness
          else                ! is also periodical! The next layer to 
           Lnext=L+1          ! the last element is the first layer! 
           if(Lnext.gt.NL) Lnext=NL
          end if              


	    DO I=1,2
	     DO J=1,2 
	      IF (Lprev.EQ.0) THEN 
	       KLprev(I,J)=C0     ! In case of 1st layer KLprev is for vacuum
	       prev_dgnl=.true.
		   END IF                
		  IF (Lprev.GT.0) THEN 	  
          	   KLprev(I,J)=Cmatrix(I,J,Lprev)
	           prev_dgnl=.not.b_Cm(Lprev)							
	       ENDIF

		 END DO
	    END DO

      DabsMatr=dreal(KLprev(1,2)-KL(1,2))*conjg((KLprev(1,2)-KL(1,2)))+
     &              (KLprev(2,1)-KL(2,1))*conjg((KLprev(2,1)-KL(2,1)))         
	
      IF (prev_dgnl.and.b_odz) then	  !
	both_dgnl=.true.				  !	 Rohlsberger roughness1
	both_NONdgnl=.false.              !
	ELSEIF (DabsMatr.lt.1.e-15)	then  !
	both_dgnl=.true.				  !  Rohlsberger roughness1
	both_NONdgnl=.false.			  !
	ELSEIF (.not.prev_dgnl.and..not.b_odz) then
	both_dgnl=.false. 				  !
      both_NONdgnl=.true.				  !Deak's roughness
	ELSE							  
	both_dgnl=.false. 				  !	Rohlsberger roughness2
      both_NONdgnl=.false.			  !
	ENDIF

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C	both_dgnl=.false. 				  !DONT FORGET TO REMOVE!!!!!!!!!
C      both_NONdgnl=.TRUE.				  !Deak's roughness
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      call roughness(both_dgnl,both_NONdgnl,roughlay(L),roughlay(Lnext),
     &QD,KLprev,KL,M)    
		END IF !Roughness

      call mat4(M,ML,ML) 
  
         endif ! VKD(L) > 0.         
      z1=z2
         end do ! L
	END DO ! while

   
C
        POINTER=POINTER+GEOSTR(2*GRP)
       end do ! GRP

       return
       end

C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine trans(QD,KL,R,L,A,B)

C *******************************************************************
C
C           Calculation of the 2x2 A and B matrixes - amplitudes of wave on the upper border
C
C ******************************************************************

       implicit none

 ! Input 
      real*8 QD     !Kz- perpendicular komponent of wave vektor (Kz=Qz*0.5)
      complex*16 KL(2,2),R(2,2),L(4,4) !
  
!   Output
      complex*16 A(2,2),B(2,2)


!Other
	complex*16 L11(2,2),L12(2,2), L21(2,2),L22(2,2),
     &             RM(2,2),RP(2,2),A1(2,2),A2(2,2),
     &             C1(2,2),C2(2,2),C1P(2,2),C2P(2,2),
     &             C1M(2,2),C2M(2,2)
      real*8 QD2

      integer    I,J
	complex*16 remove_me_later(2,2)
C   ********************Body of the subroutine**************************************

      QD2=QD*QD

C    Definition of 2x2 block matrices L21, L22 and 
C    the 2x2 I-R and I+R matrices

        do I=1,2
         do J=1,2
          L11(I,J)= 0.5D0*L(I,J)  
	    L12(I,J)= 0.5D0*L(I,J+2)
          L21(I,J)= 0.5D0*L(I+2,J)
          L22(I,J)= 0.5D0*L(I+2,J+2)
          RM(I,J) = -R(I,J)
          RP(I,J) = R(I,J)
         end do
         RM(I,I)  =1.D0-R(I,I)
         RP(I,I)  =1.D0+R(I,I)
        end do
!Now Lij are block matrixes already divided by 2, Rm=1-R, Rp=1+R
        
       A1(1,1)=(KL(1,1)/QD2)+1     !
       A1(1,2)=KL(1,2)/QD2         !A1=1+(K/Kz**2)
	 A1(2,1)=KL(2,1)/QD2         !
	 A1(2,2)=(KL(2,2)/QD2)+1     !


       call sqrt2(A1,A2)          !A2=sqrt(A1)==etha  
      call mat2(A2,A2,remove_me_later)

	call inv2(A2,A1)           !A1=etha**-1
 
      call mat2(A1,L11,C1)    !C1=(etha**-1)*L11*0.5
      call mat2(A1,L12,C2)    !C2=(etha**-1)*L12*0.5

        do I=1,2
         do J=1,2
          C1P(I,J)=L21(I,J)+C1(I,J)  !C1P=(L21+(etha**-1)*L11)*0.5
          C1M(I,J)=L21(I,J)-C1(I,J)  !C1M=(L21-(etha**-1)*L11)*0.5
          C2P(I,J)=L22(I,J)+C2(I,J)  !C2P=(L22+(etha**-1)*L12)*0.5
          C2M(I,J)=L22(I,J)-C2(I,J)  !C2M=(L22-(etha**-1)*L12)*0.5
         end do
        end do
      
	call mat2(C1P,Rm,C1)    !C1=(L21+(etha**-1)*L11)*(1-R)*0.5
      call mat2(C2P,Rp,C2)    !C2=(L22+(etha**-1)*L12)*(1+R)*0.5
	call mat2(C1M,Rm,A1)    !A1=(L21-(etha**-1)*L11)*(1-R)*0.5
	call mat2(C2M,Rp,A2)    !A2=(L22-(etha**-1)*L12)*(1+R)*0.5

        do I=1,2
         do J=1,2
          A(I,J)=C1(I,J)+C2(I,J)  !A=[L21+(etha**-1)*L11)*(1-R)+
		                        !   (L22+(etha**-1)*L12)*(1+R)]*0.5
          B(I,J)=A1(I,J)+A2(I,J)  !B=[(L21-(etha**-1)*L11)*(1-R)+
		!                           (L22-(etha**-1)*L12)*(1+R)]*0.5
         end do
        end do

	  return
        end
C ******************************************************************************
C Subroutines calculate integral (exp(ikz)*A+exp(-ikz)B)*exp(-ik'z)dz 
C in the interval from z1 to z2
C ******************************************************************************

	subroutine analytic_fourier(z1,z2,QD,K1,KL,A,B,T)
	implicit none

! Input
	real*8 z1,z2,QD,K1
      complex*16 KL(2,2),A(2,2),B(2,2)
!OutPut   
      complex*16 T(2,2)
! Other
      complex*16 Im1,b_const,iz1,iz2,DM1(2,2),K(2,2),DMA(2,2),DMB(2,2),
     &DMA1(2,2),DMB1(2,2),DMA2(2,2),DMB2(2,2),DMAS(2,2),DMBS(2,2),
     &DMAA(2,2),DMBB(2,2)
	parameter (Im1=(0.0D0,1.0D0))
	real*8 Kz2
	integer I,J

C********************Body of the subroutine

      iz1=Im1*z1
      iz2=Im1*z2
	b_const=1.0/(Im1*2.50662827)  !b_const=1/i*sqrt(2*pi)
      Kz2=QD*QD
 
      DM1(1,1)=KL(1,1)+Kz2
      DM1(1,2)=KL(1,2)                          !DM1=Ko**2+KL
      DM1(2,1)=KL(2,1)
      DM1(2,2)=KL(2,2)+Kz2 

      call sqrt2(DM1,K)   ! K=sqrt(Ko**2+KL)- z component of incoming wavevector   
                          ! in the given layer
      
      DO I=1,2
	 DO J=1,2
      DMA(I,J)=K(I,J) 
	DMB(I,J)=K(I,J) 
	 END DO
      DMA(I,I)=K(I,I)-K1                !DMA=K-K1
	DMB(I,I)=K(I,I)+K1                !DMB=K+K1
      END DO

      DO I=1,2
	 DO J=1,2
       DMA1(I,J)= iz1*DMA(I,J)       !DMA1= i*z1*(K-K1)
	 DMA2(I,J)= iz2*DMA(I,J)       !DMA1= i*z2*(K-K1)
       DMB1(I,J)=-iz1*DMB(I,J)       !DMB1=-i*z1*(K+K1)
	 DMB2(I,J)=-iz2*DMB(I,J)       !DMB2=-i*z2*(K+K1)
	 END DO
      END DO

      call exp2(DMA1)    !DMA1=exp(i*z1*(K-K1))
	call exp2(DMA2)    !DMA2=exp(i*z2*(K-K1))
	call exp2(DMB1)    !DMB1=exp(-i*z1*(K+K1))
	call exp2(DMB2)    !DMB2=exp(-i*z2*(K+K1))

      DO I=1,2
	 DO J=1,2
       DMAS(I,J)=DMA2(I,J)-DMA1(I,J) !DMAS=exp(i*z2*(K-K1))-exp(i*z1*(K-K1))
       DMBS(I,J)=DMB1(I,J)-DMB2(I,J) !DMBS=exp(-i*z1*(K+K1))-exp(-i*z2*(K+K1))
	 END DO
      END DO
      
      
      call inv2(DMA,DMAA) !DMAA=[K-K1]**-1
      call inv2(DMB,DMBB) !DMBB=[K+K1]**-1
	
!	call mat2(A,DMAA,DMA) !DMA=A*[K-K1]**-1
!	call mat2(B,DMBB,DMB) !DMB=B*[K+K1]**-1

      call mat2(DMAA,A,DMA) !DMA=([K-K1]**-1)*A
	call mat2(DMBB,B,DMB) !DMB=([K+K1]**-1)*B

	call mat2(DMAS,DMA,DMAA)
!DMAA=[exp(i*z2*(K-K1))-exp(i*z1*(K-K1))]*([K-K1]**-1)*A
      call mat2(DMBS,DMB,DMBB)
!DMBB=[exp(-i*z1*(K+K1))-exp(-i*z2*(K+K1))]*([K+K1]**-1)*B

      DO I=1,2
	 DO J=1,2
      T(I,J)=(DMAA(I,J)+DMBB(I,J))*b_const
	 END DO
      END DO
      

	return
      end


C *******************************************************************    
      subroutine  n_diffuseRough(I_DMAX,maxl,Nl,b_Cm,MAXGRP,GEOSTR,
     &                     SFi_inc,SFi_fin,k,Cmatrix,VKD,rough,Hext,
     &                     ksi,A,DWBA,Dper,Dgrp,Inc_matrix,Dif_matrix,
     &                     b_plus,Imatrix)

C*******************************************************************    
C    Subroutine calculates diffuse scattering 
C    using field of both incoming and outgoing waves in
C   the interfaces where roughness exits
C
C        Khaidukov. Budapest, oktober 2004
C

C ***************************************************************** 
      implicit none
C INPUT:      
C
      integer   maxl,GEOSTR(0:2*maxl),MAXGRP,Nl
      logical   b_plus
      real*4    SFi_inc,SFi_fin,k,VKD(maxl),
     &          ksi,rough(maxl),SQ,Hext
      complex*8 Cmatrix(2,2,maxl) 
      logical   DWBA,b_Cm(maxl)
      real*4    A

      complex*8 Imatrix(2,2)

C Other variables
      logical b_first,b_grp,b_grp1,b_first1
      integer POINTER,GRP,GRP1,LMIN,LMAX,n_rep,L,n,I_DMAX,POINTER1,
     &        LMIN1,LMAX1,n_rep1,n1,L1,ii,jj,Lprev,Lprev1,I1,I2
      real*8  DL,Qz,Qx,pi,DT,DG,Dper(maxl),Dgrp(maxl),C,DL1,
     &        z1_1,z2_1,z1_2,z2_2
 	    
      real*4      VKDMAX
      parameter  (VKDMAX=7.30416e3)

      complex*8  Inc_matrix(2,2,I_DMAX),Dif_matrix(2,2,I_DMAX),
     &           Rmatrix(2,2)	 
      complex*16 Inc1(2,2),Dif1(2,2),Inc2(2,2),Dif2(2,2),
     &           dI(2,2),Idif(2,2),dhi(2,2),dhi1(2,2)

c****** First field of outgoing wave is calculated
      SQ=2.0*k*sin(SFi_fin)   
      call n_optics(maxl,NL,b_Cm,MAXGRP,GEOSTR,SQ,Hext,Cmatrix,  
     &               VKD,rough,b_plus,.not.b_plus,Rmatrix)
      call n_depth(I_DMAX,maxl,NL,b_Cm,MAXGRP,GEOSTR,SQ,Cmatrix,  
     &               VKD,rough,Rmatrix,Dif_matrix)
			     !call field for diffuse wave
c*************************call field for diffuse wave
     	pi=atan(1.)*4. 
      Qz=dble(k*(sin(dble(SFi_fin))+sin(dble(SFi_inc))))
	                               !perpenidular component of Q
      Qx=dble(k*(cos(dble(SFi_fin))-cos(dble(SFi_inc))))
                                       !parallel component of Q


C Identification of the structure, calculation of the total thickness
C without substrate layer

       DT     =0.D0         ! total thickness
       POINTER=0 
       z1_1=0.0D0  !First 
       z2_1=0.0D0  !and the second coordinate for each layer
 	           ! for the first sum
                                    
       do GRP=1,MAXGRP      ! loop for the groups
        b_grp =.false.
        DG    =0.D0         ! thickness of the group

        LMIN  =POINTER+1
        LMAX  =POINTER+GEOSTR(2*GRP)
        n_rep =GEOSTR(2*GRP-1)
        if(VKD(LMAX).gt.VKDMAX)LMAX=LMAX-1

        do L=LMIN,LMAX
          b_grp =.true.
          DG    =DG+VKD(L)
        end do ! L

        Dper(GRP)=DG
         
C     For periodic multilayers

        if(n_rep.gt.1.and.b_grp)then
         DG=DG*dfloat(n_rep)
        end if  ! n_rep>1

        Dgrp(GRP)=DG
        DT       =DT+DG
        POINTER  =POINTER+GEOSTR(2*GRP)
       end do ! GRP

C	print *, DT
C***************** Sum over layers************
	 

C First sum over L

	Idif(1,1)=(0.D0,0.D0)
	Idif(1,2)=(0.D0,0.D0)
	Idif(2,1)=(0.D0,0.D0)
	Idif(2,2)=(0.D0,0.D0)     
	
	 
C First sum over L
       b_first     =.true.
       POINTER=0
       I1=1
       z1_1=0.0D0  !First 
       z2_1=0.0D0  !and the second coordinate for each layer
	            ! for the first sum
	 
	 
       do GRP=1,MAXGRP      ! loop for the groups
        b_grp=.false.
C
        LMIN =POINTER+1
        LMAX =POINTER+GEOSTR(2*GRP)
        n_rep=GEOSTR(2*GRP-1)
C
	n=1
   
        DO WHILE (n.LE.n_rep)
         n=n+1
         do L=LMIN,LMAX
	  DL=VKD(L)
	  z2_1=z1_1+DL
	 
	  if(VKD(L).gt.0.) then

          if (n_rep.GT.1.and.L.eq.Lmin) then  ! In case of repeating layers
	     Lprev=Lmax	                       ! we are more interested in the 
	     else                              !information about roughnesses
                              		       !in repeated ones. 
	     Lprev=L-1       			       !#Khaidukov
	     END IF

	        DO II=1,2  
                 DO JJ=1,2
               
                  IF (Lprev.GT.0) THEN 
	           dhi(II,JJ)=Cmatrix(II,JJ,Lprev)-Cmatrix(II,JJ,L)
	          ELSE !Susceptibility of air is zero
		   dhi(II,JJ)=-Cmatrix(II,JJ,L)
	          END IF
		     
                  END DO
	      	END DO

	   call field_in_z(I_DMAX,Inc_matrix,I1,DWBA,Inc1)!incident wave #1
	   call field_in_z(I_DMAX,Dif_matrix,I1,DWBA,Dif1)!diffuse wave  #1
!	          I1=I1+1
C Second sum over L1
	    b_first1=.true.
            POINTER1=0
	    I2=1
            z1_2=0.0D0  !First 
            z2_2=0.0D0  !and the second coordinate for each layer
	                  !for the second sum

            do GRP1=1,MAXGRP      ! loop for the groups
             b_grp1=.false.

	        LMIN1 =POINTER1+1
              LMAX1 =POINTER1+GEOSTR(2*GRP1)
              n_rep1=GEOSTR(2*GRP1-1)
C
	        n1=1
             DO WHILE (n1.LE.n_rep1)
	       n1=n1+1

               do L1=LMIN1,LMAX1
	        DL1=VKD(L1)
                z2_2=z1_2+DL1
	        if(VKD(L1).gt.0.) then
	                
	          call field_in_z(I_DMAX,Inc_matrix,I2,DWBA,Inc2)
                                                    !incident wave #2
	          call field_in_z(I_DMAX,Dif_matrix,I2,DWBA,Dif2)
                                                    !diffuse wave  #2
!	          I2=I2+1   
 
                  if (n_rep1.GT.1.and.L1.eq.Lmin1) then  
                                   ! In case of repeating layers
	            Lprev1=Lmax1   ! we are more interested in the 
	          else             !information about roughnesses
             		           !in repeated ones. 
	           Lprev1=L1-1     ! Khaidukov
	          END IF

	          DO II=1,2 
	 	   DO JJ=1,2
	       	    IF (Lprev1.GT.0) THEN 
		     dhi1(II,JJ)=Cmatrix(II,JJ,Lprev1)-Cmatrix(II,JJ,L1)
	            ELSE !Susceptibility of air is zero
                     dhi1(II,JJ)=-Cmatrix(II,JJ,L1)
	            END IF
		   END DO
	      	  END DO

                  call corellateRough(z1_1,z1_2,rough(L),rough(L1),
     &                                                 Qx,Qz,ksi,A,C)


	          call contributionRough(Inc1,Dif1,dhi,Inc2,Dif2,dhi1,
     &		                                                  C,dI)

		  Idif(1,1)=Idif(1,1)+dI(1,1)
		  Idif(1,2)=Idif(1,2)+dI(1,2)  
	          Idif(2,1)=Idif(2,1)+dI(2,1)
	          Idif(2,2)=Idif(2,2)+dI(2,2)

	        endif ! VKD(L1) > 0. and ksi(L1)>0
	      	I2=I2+1
                z1_2=z2_2
               end do ! L1
		  
              END DO  ! n1<=n1_rep
	      POINTER1=POINTER1+GEOSTR(2*GRP1)
	      end do ! GRP1

           end if ! VKD(L) > 0. and ksi(L)>0        
!	   print *, d1,d2
	   I1=I1+1
c          print *, z1_1,z2_1
           z1_1=z2_1
         end do ! L
	END DO  ! n<=n_rep

        POINTER=POINTER+GEOSTR(2*GRP)
       end do ! GRP
	
!      Imatrix(1,1)=r*I(1,1)/(157.91D0)  !157.91=16*pi**2
!	Imatrix(1,2)=r*I(1,2)/(157.91D0)
!	Imatrix(2,1)=r*I(2,1)/(157.91D0)
!	Imatrix(2,2)=r*I(2,2)/(157.91D0)

        Imatrix(1,1)=Idif(1,1)
	Imatrix(1,2)=Idif(1,2)
	Imatrix(2,1)=Idif(2,1)
	Imatrix(2,2)=Idif(2,2)


!	If (Imag(I).NE.0) print *, Imag(I)/Real(I)


!	Idif=abs(I)
!	Idif=cabs(I)
	return
	end

C  *****************************************************************
C    Contribution to diffuse scattering of a layer for 
C    a given polarization.      
C    *****************************************************************
	subroutine contributionRough(Inc1,Dif1,dhi,Inc2,Dif2,dhi1,
     &		                                     C,dI)
	implicit none

	complex*16 Inc1(2,2),Dif1(2,2),Inc2(2,2),Dif2(2,2),
     &ImUnit,productM(2,2),
     &productM1(2,2),dhi(2,2),dhi1(2,2),dI(2,2)

	parameter (ImUnit=(0.D0,1.D0))
	real*8 C

!Product of matrix part of intensity
	call mat2(dhi,Dif1,productM)   !productM=dhi*Dif1
	call mat2(productM,Inc1,productM) !productM=dhi*Dif1*Inc1
	call ket_from_bra(productM)	  !productM=transposed(dhi*Dif1*Inc1)

	call mat2(dhi1,Dif2,productM1)   !productM1=dhi1*Dif2
	call mat2(productM1,Inc2,productM1) !productM1=dhi1*Dif2*Inc2

	call mat2(productM,productM1,productM) 
         !productM=transposed(dhi*Dif1*Inc1)*dhi1*Dif2*Inc2

	dI(1,1)=productM(1,1)*C
        dI(1,2)=productM(1,2)*C
	dI(2,1)=productM(2,1)*C
	dI(2,2)=productM(2,2)*C

      return
      end

C  *****************************************************************
C    Correlation function of susceptibilities in  layers
C         
C   	C=... Sm diser
C  *****************************************************************
	subroutine  corellateRough(z1,z2,sigma1,sigma2,
     &			  Qx,Qz,ksi,A,C)

	implicit none
!Input parameters
	real*4 ksi,sigma1,sigma2,A
	real*8 Qx,Qz,z1,z2

!Output 
	real*8 C
!Other
	real*8 ksiD,dnm,sigma0D

	ksiD=ksi*dexp(-abs(z1-z2)/A)

      sigma0D=sqrt(sigma1*sigma2)

	dnm=((Qz*Qz*sigma0D*sigma0D+1)+(Qx*ksiD)*(Qx*ksiD))**1.5

	C=2.D0*sigma0D*sigma0D*(ksiD*ksiD/dnm)

!	C=1.
      return
      end

C  *****************************************************************
C    Field in width d1
C    
C   	
C  *****************************************************************


	subroutine field_in_z(I_DMAX,T_z,layer,DWBA,T_kz)
	implicit none

	integer	 I_DMAX,layer,I,J
	complex*16 T_kz(2,2)
      complex*8  T_z(2,2,I_DMAX)
	logical DWBA

C
 	IF (DWBA) then

	 DO I=1,2			   !
	  DO J=1,2			   !
	  T_kz(I,J)=T_z(I,J,layer)  
     	  END DO
	 END DO 



      ELSE
	T_kz(1,1)=cmplx(1.D0)         !
	T_kz(1,2)=cmplx(0.D0)		  !Born approximation
	T_kz(2,1)=cmplx(0.D0)		  !
	T_kz(2,2)=cmplx(1.D0)
	END IF
      return
      end

	subroutine ket_from_bra(matrix)
	implicit none
	complex*16 matrix(2,2),help_var


	matrix(1,1)=conjg(matrix(1,1))
	matrix(2,2)=conjg(matrix(2,2))
	help_var=conjg(matrix(1,2))
	matrix(1,2)=conjg(matrix(2,1))
	matrix(2,1)=help_var


      return
      end

C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
       subroutine n_depth(I_DMAX,maxll,NL,b_Cm,MAXGRP,GEOSTR,SQ,Cmatrix,
     &                      VKD,roughlay,Rmatrix,Tmatrix)
C
C ******************************************************************    
C
C               Subroutine calculates field on each interfaces,        
C                and stores it in Tmatrix(I), where
C              I number of the interfaces.i=1 - surface
C
C            Budapest, 2004
C
C               Khaidukov Yu. N. 
C		
C ***************************************************************** 

       

       implicit none
C   input:
       integer    NL,maxl,maxll
       parameter  (maxl=48)
C     NL: number of layers  (in period)
C   output:
       complex*16 T(2,2)
C   other variables:
       complex*16 C0,C1,CI,M(4,4),ML(4,4),KL(2,2),R(2,2)
       real*8     QD, DL
       integer    I,J,L,n_rep,GRP,LMIN,LMAX,
     &            GEOSTR(0:2*maxl),MAXGRP,POINTER,I_DMAX,I_D


       parameter  (C0=(0.D0,0.D0), C1=(1.D0,0.D0), CI=(0.D0,1.D0))
C**************************************

       complex*8  Cmatrix(2,2,maxl),Rmatrix(2,2),cz,
     &            Tmatrix(2,2,I_DMAX)
       real*4     VKD(maxl),SQ,roughlay(maxl),absCmatr,
     &            VKDMAX
       logical    b_Cm(maxl),b_odz,b_first,b_grp
       parameter  (cz=(0.0,0.0),VKDMAX=7.30416e3)

C********* Variables added by Khaidukov for neutron case
	integer Lprev,Lnext
	logical prev_dgnl,both_dgnl,both_NONdgnl
	complex*16 KLprev(2,2)
	real*8 DabsMatr 
	integer n
             
C---------------------------------------------
	NL=5 !Unused for this program variable
       
	 QD=dble(SQ)*0.5D0 

       R(1,1) = Rmatrix(1,1)
       R(1,2) = Rmatrix(1,2)
       R(2,1) = Rmatrix(2,1)
       R(2,2) = Rmatrix(2,2)
C 
       b_first     =.true.
	I_D=0

C       To set the characteristic matrix to unit matrix
        do I=1,4
         do J=1,4
          ML(I,J)=C0
         end do
         ML(I,I)=C1
        end do

       POINTER=0                                     
       do GRP=1,MAXGRP      ! loop for the groups
        b_grp=.false.
C
        LMIN =POINTER+1
        LMAX =POINTER+GEOSTR(2*GRP)
        n_rep=GEOSTR(2*GRP-1)
C
      	n=1
   
      DO WHILE (n.LE.n_rep)
	n=n+1
        do L=LMIN,LMAX 

C-------------------Store field near interface with roughness
!	IF (roughlay(L).GT.0) THEN              !Uncomment this if you want to
	    call transRough(ML,R,T)                   !store field only on the 
	    I_D=I_D+1                         
	         !---------
	          Tmatrix(1,1,I_D)=T(1,1)
                Tmatrix(1,2,I_D)=T(1,2)
                Tmatrix(2,1,I_D)=T(2,1)
                Tmatrix(2,2,I_D)=T(2,2)
	         !-------
!		END IF !Roughness                 !interfaces with roughnesses
C---------------------------------------------------------------

	 if(VKD(L).gt.0.and.VKD(L).lt.1.e30) then 
C          b_odz=.false.                              

	    b_odz=.not.b_Cm(L)
		b_grp=.true.  			 
            DL=VKD(L)
            KL(1,1)=Cmatrix(1,1,L)
            KL(1,2)=Cmatrix(1,2,L)
            KL(2,1)=Cmatrix(2,1,L)
            KL(2,2)=Cmatrix(2,2,L)
            absCmatr=real(Cmatrix(1,2,L)*conjg(Cmatrix(1,2,L))+
     &                    Cmatrix(2,1,L)*conjg(Cmatrix(2,1,L)))
           if(absCmatr.lt.1.e-15) then
		 b_odz=.true.
	     end if


          call genM(b_odz,QD,DL,KL,M)  !
C************* Roughness************
	IF (roughlay(L).GT.0) THEN
           if (n_rep.GT.1.and.L.eq.Lmin) then  ! In case of repeating layers
	     Lprev=Lmax	                       ! we are more interested in the 
	     else                              !information about roughnesses
                              		       !in repeated ones. 
	     Lprev=L-1			               !#Khaidukov
	     END IF

          if(n_rep.GT.1.and.L.eq.LMAX) then  !
           Lnext=LMIN         ! For peiodical structure the rougness
          else                ! is also periodical! The next layer to 
           Lnext=L+1          ! the last element is the first layer! 
           if(Lnext.gt.NL) Lnext=NL
          end if              


	    DO I=1,2
	     DO J=1,2 
	      IF (Lprev.EQ.0) THEN 
	       KLprev(I,J)=C0     ! In case of 1st layer KLprev is for vacuum
	       prev_dgnl=.true.
		   END IF                
		  IF (Lprev.GT.0) THEN 	  
          	   KLprev(I,J)=Cmatrix(I,J,Lprev)
	           prev_dgnl=.not.b_Cm(Lprev)							
	       ENDIF

		 END DO
	    END DO

      DabsMatr=dreal(KLprev(1,2)-KL(1,2))*conjg((KLprev(1,2)-KL(1,2)))+
     &              (KLprev(2,1)-KL(2,1))*conjg((KLprev(2,1)-KL(2,1)))         
	
      IF (prev_dgnl.and.b_odz) then	  !
	both_dgnl=.true.				  !	 Rohlsberger roughness1
	both_NONdgnl=.false.              !
	ELSEIF (DabsMatr.lt.1.e-15)	then  !
	both_dgnl=.true.				  !  Rohlsberger roughness1
	both_NONdgnl=.false.			  !
	ELSEIF (.not.prev_dgnl.and..not.b_odz) then
	both_dgnl=.false. 				  !
      both_NONdgnl=.true.				  !Deak's roughness
	ELSE							  
	both_dgnl=.false. 				  !	Rohlsberger roughness2
      both_NONdgnl=.false.			  !
	ENDIF

      call roughness(both_dgnl,both_NONdgnl,roughlay(L),roughlay(Lnext),
     &QD*0.5D0,KLprev,KL,M)
		END IF !Roughness

          call mat4(M,ML,ML) 
         endif ! VKD(L) > 0.         
         end do ! L
	END DO ! while

   
C
        POINTER=POINTER+GEOSTR(2*GRP)
       end do ! GRP

       return
       end

C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine transRough(L,R,T)

C *******************************************************************
C
C           Calculation of the 2x2 transmission matrix T
C
C ******************************************************************

       implicit none

       complex*16 L21(2,2),L22(2,2),R(2,2),L(4,4),
     &             RM(2,2),RP(2,2),T(2,2)
       integer    I,J

C    Definition of 2x2 block matrices L21, L22 and 
C    the 2x2 I-R and I+R matrices

        do I=1,2
         do J=1,2
          L21(I,J)= L(I+2,J)
          L22(I,J)= L(I+2,J+2)
          RM(I,J) = -R(I,J)
!         RM(I,J) = R(I,J)     !Khaidukov.
          RP(I,J) = R(I,J)
         end do
         RM(I,I)  =1.D0-R(I,I)
         RP(I,I)  =1.D0+R(I,I)
        end do

        call mat2(L21,RM,L21)

        call mat2(L22,RP,L22)

        do I=1,2
         do J=1,2
          T(I,J)=L21(I,J)+L22(I,J)
         end do
        end do

        return
        end













































